"""Functions for repairing a wheel."""

import ast
import base64
import hashlib
import os
import pprint
import random
import shutil
import tempfile
import typing
import zipfile
from . import patch_dll
from . import dll_list

LOAD_ORDER_FILENAME = '.load_order'

# Template for patching __init__.py so that the vendored DLLs are loaded at
# runtime. An empty triple-quoted string is placed at the beginning so that the
# comment "start delvewheel patch" does not show up when the built-in help
# system help() is invoked on the package. For Python >=3.8, we use the
# os.add_dll_directory() function so that the folder containing the vendored
# DLLs is added to the DLL search path. For Python 3.7 or lower, this function
# is unavailable, so we preload the DLLs. Whenever Python needs a vendored DLL,
# it will use the already-loaded DLL instead of searching for it.
#
# To use the template, call str.format(), passing in a random number and the
# name of the directory containing the vendored DLLs.
_patch_init_template = f"""

""\"""\"  # start delvewheel patch
def _delvewheel_init_patch_{{0}}():
    import os
    import sys
    libs_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir, {{1!r}}))
    if sys.version_info[:2] >= (3, 8):
        os.add_dll_directory(libs_dir)
    else:
        from ctypes import WinDLL
        with open(os.path.join(libs_dir, {LOAD_ORDER_FILENAME!r})) as file:
            load_order = file.read().split()
        for lib in load_order:
            WinDLL(os.path.join(libs_dir, lib))


_delvewheel_init_patch_{{0}}()
del _delvewheel_init_patch_{{0}}
# end delvewheel patch

"""

pp = pprint.PrettyPrinter(indent=4)


class WheelRepair:
    """An instance represents a wheel that can be repaired."""

    def __init__(self,
                 whl_path: str,
                 extract_dir: typing.Optional[str] = None,
                 add_dlls: typing.Optional[typing.Set[str]] = None,
                 no_dlls: typing.Optional[typing.Set[str]] = None,
                 verbose: int = 0) -> None:
        """Initialize a wheel repair object.
        whl_path: Path to the wheel to repair
        extract_dir: Directory where intermediate files are created in the
            process of repairing the wheel. If None, a temporary directory is
            created.
        dest_dir: Directory to place the repaired wheel. If None, it defaults to
            wheelhouse, relative to the current working directory.
        add_dlls: Set of lowercase DLL names to force inclusion into the wheel
        no_dlls: Set of lowercase DLL names to force exclusion from wheel
            (cannot overlap with add_dlls)
        no_mangles: Set of lowercase DLL names not to mangle
        verbose: verbosity level, 0 to 3"""
        if not os.path.isfile(whl_path):
            raise FileNotFoundError(f'{whl_path} not found')
        self._whl_path = whl_path
        self._whl_name = os.path.basename(whl_path)
        if extract_dir is None:
            # need to assign temp directory object to an attribute to prevent it
            # from being destructed
            self._extract_dir_obj = tempfile.TemporaryDirectory()
            self._extract_dir = self._extract_dir_obj.name
        else:
            self._extract_dir_obj = None
            self._extract_dir = extract_dir
        self._add_dlls = set() if add_dlls is None else add_dlls
        self._no_dlls = set() if no_dlls is None else no_dlls
        self._verbose = verbose

    @staticmethod
    def _rehash(file_path: str) -> typing.Tuple[str, int]:
        """Return (hash, size) for a file with path file_path. The hash and size
        are used by pip to verify the integrity of the contents of a wheel."""
        with open(file_path, 'rb') as file:
            contents = file.read()
            hash = base64.urlsafe_b64encode(hashlib.sha256(contents).digest()).decode('latin1').rstrip('=')
            size = len(contents)
            return hash, size

    @staticmethod
    def _hashfile(afile: typing.BinaryIO, blocksize: int = 65536, length: int = 8) -> str:
        """Hash the contents of an open file handle with SHA256. Return the
        first length characters of the hash."""
        hasher = hashlib.sha256()
        buf = afile.read(blocksize)
        while len(buf) > 0:
            hasher.update(buf)
            buf = afile.read(blocksize)
        return hasher.hexdigest()[:length]

    def _patch_init(self, init_path: str, libs_dir: str) -> None:
        """Given the path to __init__.py, create or patch the file so that
        vendored-in DLLs can be loaded at runtime.
        libs_dir is the name of the directory where DLLs are stored."""
        print(f'patching {os.path.relpath(init_path, self._extract_dir)}')

        open(init_path, 'a+').close()
        with open(init_path) as file:
            init_contents = file.read()
        node = ast.parse(init_contents)
        docstring = ast.get_docstring(node)

        rand_num = random.randint(10 ** 10, 10 ** 11 - 1)
        patch_init_contents = _patch_init_template.format(rand_num, libs_dir)

        if docstring is None:
            # prepend patch
            with open(init_path, 'w') as file:
                file.write(patch_init_contents)
                file.write(init_contents)
        else:
            # insert patch after docstring
            children = list(ast.iter_child_nodes(node))
            if len(children) == 0 or not isinstance(children[0], ast.Expr) or \
                    not isinstance(children[0].value, ast.Constant) or \
                    children[0].value.value != docstring:
                raise ValueError('Error parsing __init__.py: docstring exists but is not the first element of the parse tree')
            elif len(children) == 1:
                with open(init_path, 'a') as file:
                    file.write(patch_init_contents)
            else:
                if not init_contents.lstrip().startswith('"""'):
                    raise ValueError('Error parsing __init__.py: docstring exists but is not a triple-quoted string at the start of the file')
                docstring_start_index = init_contents.index('"""')
                docstring_end_index = init_contents.find('"""', docstring_start_index + 1) + 3
                if docstring_end_index == -1:
                    raise ValueError('Error parsing __init__.py: docstring exists but does not end with triple quotes')
                docstring_end_line = init_contents.find('\n', docstring_end_index)
                if docstring_end_line == -1:
                    docstring_end_line = len(init_contents)
                extra_text = init_contents[docstring_end_index: docstring_end_line]
                if extra_text and not extra_text.isspace():
                    raise ValueError(f'Error parsing __init__.py: extra text {extra_text!r} is on the line where the docstring ends')
                with open(init_path, 'w') as file:
                    file.write(init_contents[:docstring_end_index])
                    file.write('\n')
                    file.write(patch_init_contents)
                    file.write(init_contents[docstring_end_index:])

    def show(self) -> None:
        """Show the dependencies that the wheel has."""
        print(f'Analyzing {self._whl_name}\n')

        # extract wheel
        try:
            shutil.rmtree(self._extract_dir)
        except FileNotFoundError:
            pass
        os.makedirs(self._extract_dir)
        if self._verbose >= 1:
            print(f'extracting {self._whl_name} to {self._extract_dir}')
        with zipfile.ZipFile(self._whl_path) as whl_file:
            whl_file.extractall(self._extract_dir)

        # find dependencies
        dependency_paths = set()
        ignored_dll_names = set()
        not_found_dll_names = set()
        extension_module_paths = []
        for root, _, filenames in os.walk(self._extract_dir):
            for filename in filenames:
                if filename.endswith('.pyd'):
                    extension_module_path = os.path.join(root, filename)
                    extension_module_paths.append(extension_module_path)
                    discovered, ignored, not_found = patch_dll.get_all_needed(extension_module_path, self._add_dlls, self._no_dlls, 'ignore')
                    dependency_paths |= discovered
                    ignored_dll_names |= ignored
                    not_found_dll_names |= not_found

        dependency_paths = list(dependency_paths)
        dependency_paths.sort()
        print('The following dependent DLLs will be copied into the wheel.')
        if dependency_paths:
            for dependency_path in dependency_paths:
                print(f'    {os.path.basename(dependency_path)} ({dependency_path})')
            for not_found_dll_name in not_found_dll_names:
                print(f'    {not_found_dll_name} (Error: Not Found)')
        else:
            print('    None')

        ignored_dll_names = list(ignored_dll_names)
        ignored_dll_names.sort()
        print('\nThe following dependent DLLs will not be copied into the wheel.')
        if ignored_dll_names:
            for ignored_dll_name in ignored_dll_names:
                print(f'    {ignored_dll_name}')
        else:
            print('    None')
        if not_found_dll_names:
            print('\nWarning: At least one dependent DLL needs to be copied into the wheel but was not found.\n\n')

    def repair(self, target: str, no_mangles: set, no_mangle_all: bool, lib_sdir: str) -> None:
        """Repair the wheel in a manner similar to auditwheel.
        target is the target directory for storing the repaired wheel
        no_mangles is a set of lowercase DLL names that will not be mangled
        no_mangle_all is True if no DLL name mangling should happen at all
        lib_sdir is the suffix for the directory to store the DLLs"""
        print(f'repairing {self._whl_path}')

        # extract wheel
        try:
            shutil.rmtree(self._extract_dir)
        except FileNotFoundError:
            pass
        os.makedirs(self._extract_dir)
        if self._verbose >= 1:
            print(f'extracting {self._whl_name} to {self._extract_dir}')
        with zipfile.ZipFile(self._whl_path) as whl_file:
            whl_file.extractall(self._extract_dir)

        # find dependencies and copy them into wheel
        print('finding DLL dependencies')
        dependency_paths = set()
        ignored_dll_names = set()
        extension_module_paths = []
        for root, _, filenames in os.walk(self._extract_dir):
            for filename in filenames:
                if filename.lower().endswith('.pyd'):
                    extension_module_path = os.path.join(root, filename)
                    extension_module_paths.append(extension_module_path)
                    discovered, ignored = patch_dll.get_all_needed(extension_module_path, self._add_dlls, self._no_dlls)[:2]
                    dependency_paths |= discovered
                    ignored_dll_names |= ignored
        if not dependency_paths:
            print('no external dependencies are needed')
            return
        if self._verbose >= 1:
            print(f'External dependencies to copy into the wheel are\n{pp.pformat(set(os.path.basename(p) for p in dependency_paths))}')
            print(f'External dependencies not to copy into the wheel are\n{pp.pformat(ignored_dll_names)}')
        distribution_name = self._whl_name[:self._whl_name.index('-')]
        libs_dir_name = distribution_name + lib_sdir
        libs_dir = os.path.join(self._extract_dir, libs_dir_name)
        os.makedirs(libs_dir, exist_ok=True)
        if os.listdir(libs_dir):
            raise FileExistsError(f'The {os.path.relpath(libs_dir, self._extract_dir)} directory already exists and is nonempty')
        print(f'copying DLLs into {os.path.relpath(libs_dir, self._extract_dir)}')
        for dependency_path in dependency_paths:
            if self._verbose >= 1:
                print(f'copying {dependency_path} -> {os.path.join(libs_dir, os.path.basename(dependency_path))}')
            shutil.copy2(dependency_path, libs_dir)

        # mangle library names
        if no_mangle_all:
            print('skip mangling DLL names')
        else:
            print('mangling DLL names')
            name_mangler = {}  # dict from old name to new name
            for lib_name in os.listdir(libs_dir):
                if not any(lib_name.startswith(prefix) for prefix in dll_list.no_mangle_prefixes) and \
                        lib_name not in no_mangles:
                    root, ext = os.path.splitext(lib_name)
                    with open(os.path.join(libs_dir, lib_name), 'rb') as lib_file:
                        root = f'{root}-{self._hashfile(lib_file)}'
                    name_mangler[lib_name] = root + ext
            for extension_module_path in extension_module_paths:
                extension_module_name = os.path.basename(extension_module_path)
                if self._verbose >= 1:
                    print(f'repairing {extension_module_name} -> {extension_module_name}')
                needed = patch_dll.get_direct_mangleable_needed(extension_module_path, self._no_dlls, no_mangles)
                patch_dll.replace_needed(extension_module_path, needed, name_mangler)
            for lib_name in os.listdir(libs_dir):
                if self._verbose >= 1:
                    if lib_name in name_mangler:
                        print(f'repairing {lib_name} -> {name_mangler[lib_name]}')
                    else:
                        print(f'repairing {lib_name} -> {lib_name}')
                lib_path = os.path.join(libs_dir, lib_name)
                needed = patch_dll.get_direct_mangleable_needed(lib_path, self._no_dlls, no_mangles)
                patch_dll.replace_needed(lib_path, needed, name_mangler)
                if lib_name in name_mangler:
                    os.rename(lib_path, os.path.join(libs_dir, name_mangler[lib_name]))

        # Perform topological sort to determine the order that DLLs must be
        # loaded at runtime. We first construct a directed graph where the
        # vertices are the vendored DLLs and an edge represents a "depends-on"
        # relationship. We perform a topological sort of this graph. The reverse
        # of this topological sort then tells us what order we need to load the
        # DLLs so that all dependencies of a DLL are loaded before that DLL is
        # loaded.
        print('calculating DLL load order')
        graph = {}  # map each DLL to a set of its vendored direct dependencies
        dll_names = set(os.listdir(libs_dir))
        for dll_name in dll_names:
            dll_path = os.path.join(libs_dir, dll_name)
            # In this context, delay-loaded DLL dependencies are not true
            # dependencies because they are not necessary to get the DLL to load
            # initially. More importantly, we may get circular dependencies if
            # were were to consider delay-loaded DLLs as true dependencies.
            # For example, concrt140.dll lists msvcp140.dll in its import table,
            # while msvcp140.dll lists concrt140.dll in its delay import table.
            graph[dll_name] = patch_dll.get_direct_needed(dll_path, False) & dll_names
        rev_dll_load_order = []
        no_incoming_edge = {dll_name for dll_name in dll_names if not any(dll_name in value for value in graph.values())}
        while no_incoming_edge:
            dll_name = no_incoming_edge.pop()
            rev_dll_load_order.append(dll_name)
            while graph[dll_name]:
                dependent_dll_name = graph[dll_name].pop()
                if not any(dependent_dll_name in value for value in graph.values()):
                    no_incoming_edge.add(dependent_dll_name)
        if any(graph.values()):
            graph_leftover = {k: v for k, v in graph.items() if v}
            raise RuntimeError(f'Dependent DLLs have a circular dependency: {graph_leftover}')
        load_order_filepath = os.path.join(libs_dir, LOAD_ORDER_FILENAME)
        if os.path.exists(load_order_filepath):
            raise FileExistsError(f'{os.path.relpath(load_order_filepath, self._extract_dir)} already exists')
        with open(os.path.join(libs_dir, LOAD_ORDER_FILENAME), 'w') as file:
            file.write('\n'.join(reversed(rev_dll_load_order)))
            file.write('\n')

        # create or patch top-level __init__.py in each package to load
        # dependent DLLs from correct location at runtime
        version = self._whl_name.split('-')[1]
        for item in os.listdir(self._extract_dir):
            if os.path.isdir(os.path.join(self._extract_dir, item)) and \
                    item != f'{distribution_name}-{version}.dist-info' and \
                    item != f'{distribution_name}-{version}.data' and \
                    item != libs_dir_name:
                self._patch_init(os.path.join(self._extract_dir, item, '__init__.py'), libs_dir_name)
        purelib_dir = os.path.join(self._extract_dir, f'{distribution_name}-{version}.data', 'purelib')
        if os.path.isdir(purelib_dir):
            for item in os.listdir(purelib_dir):
                if os.path.isdir(os.path.join(purelib_dir, item)):
                    self._patch_init(os.path.join(purelib_dir, item, '__init__.py'), libs_dir_name)

        # update record file, which tracks wheel contents and their checksums
        dist_info_foldername = '-'.join(self._whl_name.split('-')[:2]) + '.dist-info'
        record_filepath = os.path.join(self._extract_dir, dist_info_foldername, 'RECORD')
        print(f'updating {os.path.join(dist_info_foldername, "RECORD")}')
        filepath_list = []
        for root, _, files in os.walk(self._extract_dir):
            for file in files:
                filepath_list.append(os.path.join(root, file))
        with open(record_filepath, 'w') as record_file:
            for file_path in filepath_list:
                if file_path == record_filepath:
                    record_file.write(os.path.relpath(record_filepath, self._extract_dir).replace('\\', '/'))
                    record_file.write(',,\n')
                else:
                    record_line = '{},sha256={},{}\n'.format(os.path.relpath(file_path, self._extract_dir).replace('\\', '/'), *self._rehash(file_path))
                    record_file.write(record_line)

        # repackage wheel
        print('repackaging wheel')
        os.makedirs(target, exist_ok=True)
        whl_dest_path = os.path.join(target, self._whl_name)
        with zipfile.ZipFile(whl_dest_path, 'w', zipfile.ZIP_DEFLATED) as whl_file:
            for file_path in filepath_list:
                relpath = os.path.relpath(file_path, self._extract_dir)
                if self._verbose >= 1:
                    print(f'adding {relpath}')
                whl_file.write(file_path, relpath)
        print(f'fixed wheel written to {os.path.abspath(whl_dest_path)}\n')



"""This file was generated by generate_dll_list.py. Do not edit."""
import re

# ignore_names_32 is a set containing the lowercase names of all DLLs that can
# be assumed to be present on 32-bit Windows 7 or later. The set is generated
# from the file SysWOW64.txt, which is a list of all files with extension .dll
# or .drv found in C:\Windows\SysWOW64 on a vanilla Windows 7 Ultimate SP1
# 64-bit installation.
ignore_names_32 = {
    'aaclient.dll',
    'accessibilitycpl.dll',
    'acctres.dll',
    'acledit.dll',
    'aclui.dll',
    'acppage.dll',
    'actioncenter.dll',
    'actioncentercpl.dll',
    'activeds.dll',
    'actxprxy.dll',
    'admtmpl.dll',
    'adprovider.dll',
    'adsldp.dll',
    'adsldpc.dll',
    'adsmsext.dll',
    'adsnt.dll',
    'adtschema.dll',
    'advapi32.dll',
    'advpack.dll',
    'aecache.dll',
    'aeevts.dll',
    'alttab.dll',
    'amstream.dll',
    'amxread.dll',
    'apds.dll',
    'api-ms-win-core-file-l1-2-0.dll',
    'api-ms-win-core-file-l2-1-0.dll',
    'api-ms-win-core-localization-l1-2-0.dll',
    'api-ms-win-core-processthreads-l1-1-1.dll',
    'api-ms-win-core-synch-l1-2-0.dll',
    'api-ms-win-core-timezone-l1-1-0.dll',
    'api-ms-win-core-xstate-l2-1-0.dll',
    'api-ms-win-crt-conio-l1-1-0.dll',
    'api-ms-win-crt-convert-l1-1-0.dll',
    'api-ms-win-crt-environment-l1-1-0.dll',
    'api-ms-win-crt-filesystem-l1-1-0.dll',
    'api-ms-win-crt-heap-l1-1-0.dll',
    'api-ms-win-crt-locale-l1-1-0.dll',
    'api-ms-win-crt-math-l1-1-0.dll',
    'api-ms-win-crt-multibyte-l1-1-0.dll',
    'api-ms-win-crt-private-l1-1-0.dll',
    'api-ms-win-crt-process-l1-1-0.dll',
    'api-ms-win-crt-runtime-l1-1-0.dll',
    'api-ms-win-crt-stdio-l1-1-0.dll',
    'api-ms-win-crt-string-l1-1-0.dll',
    'api-ms-win-crt-time-l1-1-0.dll',
    'api-ms-win-crt-utility-l1-1-0.dll',
    'apilogen.dll',
    'apircl.dll',
    'apisetschema.dll',
    'apphelp.dll',
    'apphlpdm.dll',
    'appidapi.dll',
    'appidpolicyengineapi.dll',
    'appmgmts.dll',
    'appmgr.dll',
    'apss.dll',
    'asferror.dll',
    'aspnet_counters.dll',
    'asycfilt.dll',
    'atl.dll',
    'atmfd.dll',
    'atmlib.dll',
    'audiodev.dll',
    'audioeng.dll',
    'audiokse.dll',
    'audioses.dll',
    'auditnativesnapin.dll',
    'auditpolicygpinterop.dll',
    'auditpolmsg.dll',
    'authfwcfg.dll',
    'authfwgp.dll',
    'authfwsnapin.dll',
    'authfwwizfwk.dll',
    'authui.dll',
    'authz.dll',
    'autoplay.dll',
    'auxiliarydisplayapi.dll',
    'auxiliarydisplaycpl.dll',
    'avicap32.dll',
    'avifil32.dll',
    'avrt.dll',
    'azroles.dll',
    'azroleui.dll',
    'azsqlext.dll',
    'basecsp.dll',
    'batmeter.dll',
    'bcrypt.dll',
    'bcryptprimitives.dll',
    'bidispl.dll',
    'biocredprov.dll',
    'bitsperf.dll',
    'bitsprx2.dll',
    'bitsprx3.dll',
    'bitsprx4.dll',
    'bitsprx5.dll',
    'bitsprx6.dll',
    'blackbox.dll',
    'bootvid.dll',
    'browcli.dll',
    'browseui.dll',
    'btpanui.dll',
    'bwcontexthandler.dll',
    'bwunpairelevated.dll',
    'cabinet.dll',
    'cabview.dll',
    'capiprovider.dll',
    'capisp.dll',
    'catsrv.dll',
    'catsrvps.dll',
    'catsrvut.dll',
    'cca.dll',
    'cdosys.dll',
    'certcli.dll',
    'certcredprovider.dll',
    'certenc.dll',
    'certenroll.dll',
    'certenrollui.dll',
    'certmgr.dll',
    'certpoleng.dll',
    'cewmdm.dll',
    'cfgbkend.dll',
    'cfgmgr32.dll',
    'chsbrkr.dll',
    'chtbrkr.dll',
    'chxreadingstringime.dll',
    'cic.dll',
    'clb.dll',
    'clbcatq.dll',
    'clfsw32.dll',
    'cliconfg.dll',
    'clusapi.dll',
    'cmcfg32.dll',
    'cmdial32.dll',
    'cmicryptinstall.dll',
    'cmifw.dll',
    'cmipnpinstall.dll',
    'cmlua.dll',
    'cmpbk32.dll',
    'cmstplua.dll',
    'cmutil.dll',
    'cngaudit.dll',
    'cngprovider.dll',
    'cnvfat.dll',
    'colbact.dll',
    'colorcnv.dll',
    'colorui.dll',
    'comcat.dll',
    'comctl32.dll',
    'comdlg32.dll',
    'compobj.dll',
    'compstui.dll',
    'comrepl.dll',
    'comres.dll',
    'comsnap.dll',
    'comsvcs.dll',
    'comuid.dll',
    'connect.dll',
    'console.dll',
    'cpfilters.dll',
    'credssp.dll',
    'credui.dll',
    'crtdll.dll',
    'crypt32.dll',
    'cryptbase.dll',
    'cryptdlg.dll',
    'cryptdll.dll',
    'cryptext.dll',
    'cryptnet.dll',
    'cryptsp.dll',
    'cryptsvc.dll',
    'cryptui.dll',
    'cryptxml.dll',
    'cscapi.dll',
    'cscdll.dll',
    'cscobj.dll',
    'ctl3d32.dll',
    'c_g18030.dll',
    'c_is2022.dll',
    'c_iscii.dll',
    'd2d1.dll',
    'd3d10.dll',
    'd3d10core.dll',
    'd3d10level9.dll',
    'd3d10warp.dll',
    'd3d10_1.dll',
    'd3d10_1core.dll',
    'd3d11.dll',
    'd3d8.dll',
    'd3d8thk.dll',
    'd3d9.dll',
    'd3dcompiler_47.dll',
    'd3dim.dll',
    'd3dim700.dll',
    'd3dramp.dll',
    'd3dxof.dll',
    'dataclen.dll',
    'davclnt.dll',
    'davhlpr.dll',
    'dbgeng.dll',
    'dbghelp.dll',
    'dbnetlib.dll',
    'dbnmpntw.dll',
    'dciman32.dll',
    'ddaclsys.dll',
    'ddoiproxy.dll',
    'ddores.dll',
    'ddraw.dll',
    'ddrawex.dll',
    'defaultlocationcpl.dll',
    'deskadp.dll',
    'deskmon.dll',
    'deskperf.dll',
    'devenum.dll',
    'devicecenter.dll',
    'devicedisplaystatusmanager.dll',
    'devicemetadataparsers.dll',
    'devicepairing.dll',
    'devicepairingfolder.dll',
    'devicepairinghandler.dll',
    'devicepairingproxy.dll',
    'deviceuxres.dll',
    'devmgr.dll',
    'devobj.dll',
    'devrtl.dll',
    'dfscli.dll',
    'dfshim.dll',
    'dfsshlex.dll',
    'dhcpcmonitor.dll',
    'dhcpcore.dll',
    'dhcpcore6.dll',
    'dhcpcsvc.dll',
    'dhcpcsvc6.dll',
    'dhcpqec.dll',
    'dhcpsapi.dll',
    'difxapi.dll',
    'dimsjob.dll',
    'dimsroam.dll',
    'dinput.dll',
    'dinput8.dll',
    'diskcopy.dll',
    'dispex.dll',
    'display.dll',
    'dmband.dll',
    'dmcompos.dll',
    'dmdlgs.dll',
    'dmdskmgr.dll',
    'dmdskres.dll',
    'dmdskres2.dll',
    'dmime.dll',
    'dmintf.dll',
    'dmloader.dll',
    'dmocx.dll',
    'dmrc.dll',
    'dmscript.dll',
    'dmstyle.dll',
    'dmsynth.dll',
    'dmusic.dll',
    'dmutil.dll',
    'dmvdsitf.dll',
    'dnsapi.dll',
    'dnscmmc.dll',
    'docprop.dll',
    'dot3api.dll',
    'dot3cfg.dll',
    'dot3dlg.dll',
    'dot3gpclnt.dll',
    'dot3gpui.dll',
    'dot3hc.dll',
    'dot3msm.dll',
    'dot3ui.dll',
    'dpapiprovider.dll',
    'dplayx.dll',
    'dpmodemx.dll',
    'dpnaddr.dll',
    'dpnathlp.dll',
    'dpnet.dll',
    'dpnhpast.dll',
    'dpnhupnp.dll',
    'dpnlobby.dll',
    'dpwsockx.dll',
    'dpx.dll',
    'drmmgrtn.dll',
    'drmv2clt.dll',
    'drprov.dll',
    'drt.dll',
    'drtprov.dll',
    'drttransport.dll',
    'drvstore.dll',
    'ds32gt.dll',
    'dsauth.dll',
    'dsdmo.dll',
    'dshowrdpfilter.dll',
    'dskquota.dll',
    'dskquoui.dll',
    'dsound.dll',
    'dsprop.dll',
    'dsquery.dll',
    'dsrole.dll',
    'dssec.dll',
    'dssenh.dll',
    'dsuiext.dll',
    'dswave.dll',
    'dtsh.dll',
    'dui70.dll',
    'duser.dll',
    'dwmapi.dll',
    'dwmcore.dll',
    'dwrite.dll',
    'dxdiagn.dll',
    'dxgi.dll',
    'dxmasf.dll',
    'dxptaskringtone.dll',
    'dxptasksync.dll',
    'dxtmsft.dll',
    'dxtrans.dll',
    'dxva2.dll',
    'eapp3hst.dll',
    'eappcfg.dll',
    'eappgnui.dll',
    'eapphost.dll',
    'eappprxy.dll',
    'eapqec.dll',
    'efsadu.dll',
    'efscore.dll',
    'efsutil.dll',
    'ehstorapi.dll',
    'ehstorpwdmgr.dll',
    'ehstorshell.dll',
    'els.dll',
    'elscore.dll',
    'elshyph.dll',
    'elslad.dll',
    'elstrans.dll',
    'encapi.dll',
    'encdec.dll',
    'eqossnap.dll',
    'es.dll',
    'esent.dll',
    'esentprf.dll',
    'eventcls.dll',
    'evr.dll',
    'explorerframe.dll',
    'expsrv.dll',
    'f3ahvoas.dll',
    'faultrep.dll',
    'fdbth.dll',
    'fdbthproxy.dll',
    'fde.dll',
    'fdeploy.dll',
    'fdpnp.dll',
    'fdproxy.dll',
    'fdssdp.dll',
    'fdwcn.dll',
    'fdwnet.dll',
    'fdwsd.dll',
    'feclient.dll',
    'filemgmt.dll',
    'findnetprinters.dll',
    'firewallapi.dll',
    'firewallcontrolpanel.dll',
    'fltlib.dll',
    'fmifs.dll',
    'fms.dll',
    'fontext.dll',
    'fontsub.dll',
    'fphc.dll',
    'framedyn.dll',
    'framedynos.dll',
    'fthsvc.dll',
    'fundisc.dll',
    'fwcfg.dll',
    'fwpuclnt.dll',
    'fwremotesvr.dll',
    'fxsapi.dll',
    'fxscom.dll',
    'fxscomex.dll',
    'fxsext32.dll',
    'fxsresm.dll',
    'fxsxp32.dll',
    'gameux.dll',
    'gameuxlegacygdfs.dll',
    'gcdef.dll',
    'gdi32.dll',
    'getuname.dll',
    'glmf32.dll',
    'glu32.dll',
    'gpapi.dll',
    'gpedit.dll',
    'gpprefcl.dll',
    'gpprnext.dll',
    'gpscript.dll',
    'gptext.dll',
    'hbaapi.dll',
    'hcproviders.dll',
    'helppaneproxy.dll',
    'hgcpl.dll',
    'hhsetup.dll',
    'hid.dll',
    'hidserv.dll',
    'hlink.dll',
    'hnetcfg.dll',
    'hnetmon.dll',
    'httpapi.dll',
    'htui.dll',
    'ias.dll',
    'iasacct.dll',
    'iasads.dll',
    'iasdatastore.dll',
    'iashlpr.dll',
    'iasmigplugin.dll',
    'iasnap.dll',
    'iaspolcy.dll',
    'iasrad.dll',
    'iasrecst.dll',
    'iassam.dll',
    'iassdo.dll',
    'iassvcs.dll',
    'icardie.dll',
    'icardres.dll',
    'iccvid.dll',
    'icm32.dll',
    'icmp.dll',
    'icmui.dll',
    'iconcodecservice.dll',
    'icsigd.dll',
    'idndl.dll',
    'idstore.dll',
    'ieadvpack.dll',
    'ieapfltr.dll',
    'iedkcs32.dll',
    'ieetwproxystub.dll',
    'ieframe.dll',
    'iepeers.dll',
    'iernonce.dll',
    'iertutil.dll',
    'iesetup.dll',
    'iesysprep.dll',
    'ieui.dll',
    'ifmon.dll',
    'ifsutil.dll',
    'ifsutilx.dll',
    'imagehlp.dll',
    'imageres.dll',
    'imagesp1.dll',
    'imapi.dll',
    'imapi2.dll',
    'imapi2fs.dll',
    'imgutil.dll',
    'imjp10k.dll',
    'imm32.dll',
    'inetcomm.dll',
    'inetmib1.dll',
    'inetres.dll',
    'infocardapi.dll',
    'inked.dll',
    'input.dll',
    'inseng.dll',
    'iologmsg.dll',
    'ipbusenumproxy.dll',
    'iphlpapi.dll',
    'iprop.dll',
    'iprtprio.dll',
    'iprtrmgr.dll',
    'ipsecsnp.dll',
    'ipsmsnap.dll',
    'ir32_32.dll',
    'ir41_qc.dll',
    'ir41_qcx.dll',
    'ir50_32.dll',
    'ir50_qc.dll',
    'ir50_qcx.dll',
    'irclass.dll',
    'iscsicpl.dll',
    'iscsidsc.dll',
    'iscsied.dll',
    'iscsium.dll',
    'iscsiwmi.dll',
    'itircl.dll',
    'itss.dll',
    'itvdata.dll',
    'iyuv_32.dll',
    'javascriptcollectionagent.dll',
    'jscript.dll',
    'jscript9.dll',
    'jscript9diag.dll',
    'jsintl.dll',
    'jsproxy.dll',
    'kbd101.dll',
    'kbd101a.dll',
    'kbd101b.dll',
    'kbd101c.dll',
    'kbd103.dll',
    'kbd106.dll',
    'kbd106n.dll',
    'kbda1.dll',
    'kbda2.dll',
    'kbda3.dll',
    'kbdal.dll',
    'kbdarme.dll',
    'kbdarmw.dll',
    'kbdax2.dll',
    'kbdaze.dll',
    'kbdazel.dll',
    'kbdbash.dll',
    'kbdbe.dll',
    'kbdbene.dll',
    'kbdbgph.dll',
    'kbdbgph1.dll',
    'kbdbhc.dll',
    'kbdblr.dll',
    'kbdbr.dll',
    'kbdbu.dll',
    'kbdbulg.dll',
    'kbdca.dll',
    'kbdcan.dll',
    'kbdcr.dll',
    'kbdcz.dll',
    'kbdcz1.dll',
    'kbdcz2.dll',
    'kbdda.dll',
    'kbddiv1.dll',
    'kbddiv2.dll',
    'kbddv.dll',
    'kbdes.dll',
    'kbdest.dll',
    'kbdfa.dll',
    'kbdfc.dll',
    'kbdfi.dll',
    'kbdfi1.dll',
    'kbdfo.dll',
    'kbdfr.dll',
    'kbdgae.dll',
    'kbdgeo.dll',
    'kbdgeoer.dll',
    'kbdgeoqw.dll',
    'kbdgkl.dll',
    'kbdgr.dll',
    'kbdgr1.dll',
    'kbdgrlnd.dll',
    'kbdhau.dll',
    'kbdhe.dll',
    'kbdhe220.dll',
    'kbdhe319.dll',
    'kbdheb.dll',
    'kbdhela2.dll',
    'kbdhela3.dll',
    'kbdhept.dll',
    'kbdhu.dll',
    'kbdhu1.dll',
    'kbdibm02.dll',
    'kbdibo.dll',
    'kbdic.dll',
    'kbdinasa.dll',
    'kbdinbe1.dll',
    'kbdinbe2.dll',
    'kbdinben.dll',
    'kbdindev.dll',
    'kbdinguj.dll',
    'kbdinhin.dll',
    'kbdinkan.dll',
    'kbdinmal.dll',
    'kbdinmar.dll',
    'kbdinori.dll',
    'kbdinpun.dll',
    'kbdintam.dll',
    'kbdintel.dll',
    'kbdinuk2.dll',
    'kbdir.dll',
    'kbdit.dll',
    'kbdit142.dll',
    'kbdiulat.dll',
    'kbdjpn.dll',
    'kbdkaz.dll',
    'kbdkhmr.dll',
    'kbdkor.dll',
    'kbdkyr.dll',
    'kbdla.dll',
    'kbdlao.dll',
    'kbdlk41a.dll',
    'kbdlt.dll',
    'kbdlt1.dll',
    'kbdlt2.dll',
    'kbdlv.dll',
    'kbdlv1.dll',
    'kbdmac.dll',
    'kbdmacst.dll',
    'kbdmaori.dll',
    'kbdmlt47.dll',
    'kbdmlt48.dll',
    'kbdmon.dll',
    'kbdmonmo.dll',
    'kbdne.dll',
    'kbdnec.dll',
    'kbdnec95.dll',
    'kbdnecat.dll',
    'kbdnecnt.dll',
    'kbdnepr.dll',
    'kbdno.dll',
    'kbdno1.dll',
    'kbdnso.dll',
    'kbdpash.dll',
    'kbdpl.dll',
    'kbdpl1.dll',
    'kbdpo.dll',
    'kbdro.dll',
    'kbdropr.dll',
    'kbdrost.dll',
    'kbdru.dll',
    'kbdru1.dll',
    'kbdsf.dll',
    'kbdsg.dll',
    'kbdsl.dll',
    'kbdsl1.dll',
    'kbdsmsfi.dll',
    'kbdsmsno.dll',
    'kbdsn1.dll',
    'kbdsorex.dll',
    'kbdsors1.dll',
    'kbdsorst.dll',
    'kbdsp.dll',
    'kbdsw.dll',
    'kbdsw09.dll',
    'kbdsyr1.dll',
    'kbdsyr2.dll',
    'kbdtajik.dll',
    'kbdtat.dll',
    'kbdth0.dll',
    'kbdth1.dll',
    'kbdth2.dll',
    'kbdth3.dll',
    'kbdtiprc.dll',
    'kbdtuf.dll',
    'kbdtuq.dll',
    'kbdturme.dll',
    'kbdughr.dll',
    'kbdughr1.dll',
    'kbduk.dll',
    'kbdukx.dll',
    'kbdur.dll',
    'kbdur1.dll',
    'kbdurdu.dll',
    'kbdus.dll',
    'kbdusa.dll',
    'kbdusl.dll',
    'kbdusr.dll',
    'kbdusx.dll',
    'kbduzb.dll',
    'kbdvntc.dll',
    'kbdwol.dll',
    'kbdyak.dll',
    'kbdyba.dll',
    'kbdycc.dll',
    'kbdycl.dll',
    'kerberos.dll',
    'kernel32.dll',
    'kernelbase.dll',
    'keyiso.dll',
    'keymgr.dll',
    'korwbrkr.dll',
    'ksuser.dll',
    'ktmw32.dll',
    'l2gpstore.dll',
    'l2nacp.dll',
    'l2sechc.dll',
    'laprxy.dll',
    'licmgr10.dll',
    'linkinfo.dll',
    'loadperf.dll',
    'localsec.dll',
    'locationapi.dll',
    'loghours.dll',
    'logoncli.dll',
    'lpk.dll',
    'lsmproxy.dll',
    'luainstall.dll',
    'lz32.dll',
    'magnification.dll',
    'mapi32.dll',
    'mapistub.dll',
    'mcewmdrmndbootstrap.dll',
    'mciavi32.dll',
    'mcicda.dll',
    'mciqtz32.dll',
    'mciseq.dll',
    'mciwave.dll',
    'mctres.dll',
    'mdminst.dll',
    'mediametadatahandler.dll',
    'mf.dll',
    'mf3216.dll',
    'mfaacenc.dll',
    'mfc40.dll',
    'mfc40u.dll',
    'mfc42.dll',
    'mfc42u.dll',
    'mfcsubs.dll',
    'mfds.dll',
    'mfdvdec.dll',
    'mferror.dll',
    'mfh264enc.dll',
    'mfmjpegdec.dll',
    'mfplat.dll',
    'mfplay.dll',
    'mfps.dll',
    'mfreadwrite.dll',
    'mfvdsp.dll',
    'mfwmaaec.dll',
    'mgmtapi.dll',
    'midimap.dll',
    'migisol.dll',
    'miguiresource.dll',
    'mimefilt.dll',
    'mlang.dll',
    'mmcbase.dll',
    'mmci.dll',
    'mmcico.dll',
    'mmcndmgr.dll',
    'mmcshext.dll',
    'mmdevapi.dll',
    'mmres.dll',
    'modemui.dll',
    'moricons.dll',
    'mp3dmod.dll',
    'mp43decd.dll',
    'mp4sdecd.dll',
    'mpg4decd.dll',
    'mpr.dll',
    'mprapi.dll',
    'mprddm.dll',
    'mprdim.dll',
    'mprmsg.dll',
    'msaatext.dll',
    'msac3enc.dll',
    'msacm32.dll',
    'msafd.dll',
    'msasn1.dll',
    'msaudite.dll',
    'mscandui.dll',
    'mscat32.dll',
    'msclmd.dll',
    'mscms.dll',
    'mscoree.dll',
    'mscorier.dll',
    'mscories.dll',
    'mscpx32r.dll',
    'mscpxl32.dll',
    'msctf.dll',
    'msctfmonitor.dll',
    'msctfp.dll',
    'msctfui.dll',
    'msdadiag.dll',
    'msdart.dll',
    'msdelta.dll',
    'msdmo.dll',
    'msdrm.dll',
    'msdtcprx.dll',
    'msdtcuiu.dll',
    'msdtcvsp1res.dll',
    'msexch40.dll',
    'msexcl40.dll',
    'msfeeds.dll',
    'msfeedsbs.dll',
    'msftedit.dll',
    'mshtml.dll',
    'mshtmldac.dll',
    'mshtmled.dll',
    'mshtmler.dll',
    'mshtmlmedia.dll',
    'msi.dll',
    'msidcrl30.dll',
    'msident.dll',
    'msidle.dll',
    'msidntld.dll',
    'msieftp.dll',
    'msihnd.dll',
    'msiltcfg.dll',
    'msimg32.dll',
    'msimsg.dll',
    'msimtf.dll',
    'msisip.dll',
    'msjet40.dll',
    'msjetoledb40.dll',
    'msjint40.dll',
    'msjter40.dll',
    'msjtes40.dll',
    'msls31.dll',
    'msltus40.dll',
    'msmpeg2adec.dll',
    'msmpeg2enc.dll',
    'msmpeg2vdec.dll',
    'msnetobj.dll',
    'msobjs.dll',
    'msoeacct.dll',
    'msoert2.dll',
    'msorc32r.dll',
    'msorcl32.dll',
    'mspatcha.dll',
    'mspbde40.dll',
    'msports.dll',
    'msrating.dll',
    'msrd2x40.dll',
    'msrd3x40.dll',
    'msrdc.dll',
    'msrdpwebaccess.dll',
    'msrepl40.dll',
    'msrle32.dll',
    'msscntrs.dll',
    'msscp.dll',
    'mssha.dll',
    'msshavmsg.dll',
    'msshooks.dll',
    'mssign32.dll',
    'mssip32.dll',
    'mssitlb.dll',
    'mssph.dll',
    'mssphtb.dll',
    'mssprxy.dll',
    'mssrch.dll',
    'mssvp.dll',
    'msswch.dll',
    'mstask.dll',
    'mstext40.dll',
    'mstscax.dll',
    'msutb.dll',
    'msv1_0.dll',
    'msvbvm60.dll',
    'msvcirt.dll',
    'msvcp110_clr0400.dll',
    'msvcp120_clr0400.dll',
    'msvcp60.dll',
    'msvcr100_clr0400.dll',
    'msvcr110_clr0400.dll',
    'msvcr120_clr0400.dll',
    'msvcrt.dll',
    'msvcrt20.dll',
    'msvcrt40.dll',
    'msvfw32.dll',
    'msvidc32.dll',
    'msvidctl.dll',
    'mswdat10.dll',
    'mswmdm.dll',
    'mswsock.dll',
    'mswstr10.dll',
    'msxbde40.dll',
    'msxml3.dll',
    'msxml3r.dll',
    'msxml6.dll',
    'msxml6r.dll',
    'msyuv.dll',
    'mtxclu.dll',
    'mtxdm.dll',
    'mtxex.dll',
    'mtxlegih.dll',
    'mtxoci.dll',
    'muifontsetup.dll',
    'mycomput.dll',
    'mydocs.dll',
    'napcrypt.dll',
    'napdsnap.dll',
    'naphlpr.dll',
    'napinsp.dll',
    'napipsec.dll',
    'napmontr.dll',
    'nativehooks.dll',
    'naturallanguage6.dll',
    'ncdprop.dll',
    'nci.dll',
    'ncobjapi.dll',
    'ncrypt.dll',
    'ncryptui.dll',
    'ncsi.dll',
    'nddeapi.dll',
    'ndfapi.dll',
    'ndfetw.dll',
    'ndfhcdiscovery.dll',
    'ndiscapcfg.dll',
    'ndishc.dll',
    'ndproxystub.dll',
    'negoexts.dll',
    'netapi32.dll',
    'netbios.dll',
    'netcenter.dll',
    'netcfgx.dll',
    'netcorehc.dll',
    'netdiagfx.dll',
    'netevent.dll',
    'netfxperf.dll',
    'neth.dll',
    'netid.dll',
    'netiohlp.dll',
    'netjoin.dll',
    'netlogon.dll',
    'netmsg.dll',
    'netplwiz.dll',
    'netprof.dll',
    'netprofm.dll',
    'netshell.dll',
    'netutils.dll',
    'networkexplorer.dll',
    'networkitemfactory.dll',
    'networkmap.dll',
    'newdev.dll',
    'nlaapi.dll',
    'nlhtml.dll',
    'nlmgp.dll',
    'nlmsprep.dll',
    'nlsbres.dll',
    'nlsdata0000.dll',
    'nlsdata0001.dll',
    'nlsdata0002.dll',
    'nlsdata0003.dll',
    'nlsdata0007.dll',
    'nlsdata0009.dll',
    'nlsdata000a.dll',
    'nlsdata000c.dll',
    'nlsdata000d.dll',
    'nlsdata000f.dll',
    'nlsdata0010.dll',
    'nlsdata0011.dll',
    'nlsdata0013.dll',
    'nlsdata0018.dll',
    'nlsdata0019.dll',
    'nlsdata001a.dll',
    'nlsdata001b.dll',
    'nlsdata001d.dll',
    'nlsdata0020.dll',
    'nlsdata0021.dll',
    'nlsdata0022.dll',
    'nlsdata0024.dll',
    'nlsdata0026.dll',
    'nlsdata0027.dll',
    'nlsdata002a.dll',
    'nlsdata0039.dll',
    'nlsdata003e.dll',
    'nlsdata0045.dll',
    'nlsdata0046.dll',
    'nlsdata0047.dll',
    'nlsdata0049.dll',
    'nlsdata004a.dll',
    'nlsdata004b.dll',
    'nlsdata004c.dll',
    'nlsdata004e.dll',
    'nlsdata0414.dll',
    'nlsdata0416.dll',
    'nlsdata0816.dll',
    'nlsdata081a.dll',
    'nlsdata0c1a.dll',
    'nlsdl.dll',
    'nlslexicons0001.dll',
    'nlslexicons0002.dll',
    'nlslexicons0003.dll',
    'nlslexicons0007.dll',
    'nlslexicons0009.dll',
    'nlslexicons000a.dll',
    'nlslexicons000c.dll',
    'nlslexicons000d.dll',
    'nlslexicons000f.dll',
    'nlslexicons0010.dll',
    'nlslexicons0011.dll',
    'nlslexicons0013.dll',
    'nlslexicons0018.dll',
    'nlslexicons0019.dll',
    'nlslexicons001a.dll',
    'nlslexicons001b.dll',
    'nlslexicons001d.dll',
    'nlslexicons0020.dll',
    'nlslexicons0021.dll',
    'nlslexicons0022.dll',
    'nlslexicons0024.dll',
    'nlslexicons0026.dll',
    'nlslexicons0027.dll',
    'nlslexicons002a.dll',
    'nlslexicons0039.dll',
    'nlslexicons003e.dll',
    'nlslexicons0045.dll',
    'nlslexicons0046.dll',
    'nlslexicons0047.dll',
    'nlslexicons0049.dll',
    'nlslexicons004a.dll',
    'nlslexicons004b.dll',
    'nlslexicons004c.dll',
    'nlslexicons004e.dll',
    'nlslexicons0414.dll',
    'nlslexicons0416.dll',
    'nlslexicons0816.dll',
    'nlslexicons081a.dll',
    'nlslexicons0c1a.dll',
    'nlsmodels0011.dll',
    'normaliz.dll',
    'npmproxy.dll',
    'nshhttp.dll',
    'nshipsec.dll',
    'nshwfp.dll',
    'nsi.dll',
    'ntdll.dll',
    'ntdsapi.dll',
    'ntlanman.dll',
    'ntlanui2.dll',
    'ntmarta.dll',
    'ntprint.dll',
    'ntshrui.dll',
    'ntvdm64.dll',
    'objsel.dll',
    'occache.dll',
    'ocsetapi.dll',
    'odbc32.dll',
    'odbc32gt.dll',
    'odbcbcp.dll',
    'odbcconf.dll',
    'odbccp32.dll',
    'odbccr32.dll',
    'odbccu32.dll',
    'odbcint.dll',
    'odbcji32.dll',
    'odbcjt32.dll',
    'odbctrac.dll',
    'oddbse32.dll',
    'odexl32.dll',
    'odfox32.dll',
    'odpdx32.dll',
    'odtext32.dll',
    'offfilt.dll',
    'ogldrv.dll',
    'ole2.dll',
    'ole2disp.dll',
    'ole2nls.dll',
    'ole32.dll',
    'oleacc.dll',
    'oleacchooks.dll',
    'oleaccrc.dll',
    'oleaut32.dll',
    'olecli32.dll',
    'oledlg.dll',
    'oleprn.dll',
    'olepro32.dll',
    'oleres.dll',
    'olesvr32.dll',
    'olethk32.dll',
    'onex.dll',
    'onexui.dll',
    'onlineidcpl.dll',
    'oobefldr.dll',
    'opcservices.dll',
    'opengl32.dll',
    'osbaseln.dll',
    'osuninst.dll',
    'p2p.dll',
    'p2pcollab.dll',
    'p2pgraph.dll',
    'p2pnetsh.dll',
    'packager.dll',
    'panmap.dll',
    'pautoenr.dll',
    'pcaui.dll',
    'pcwum.dll',
    'pdh.dll',
    'pdhui.dll',
    'peerdist.dll',
    'peerdistsh.dll',
    'perfcentercpl.dll',
    'perfctrs.dll',
    'perfdisk.dll',
    'perfnet.dll',
    'perfos.dll',
    'perfproc.dll',
    'perfts.dll',
    'photometadatahandler.dll',
    'photowiz.dll',
    'pid.dll',
    'pidgenx.dll',
    'pifmgr.dll',
    'pku2u.dll',
    'pla.dll',
    'playsndsrv.dll',
    'pmcsnap.dll',
    'pngfilt.dll',
    'pnidui.dll',
    'pnpsetup.dll',
    'pnrpnsp.dll',
    'polstore.dll',
    'portabledeviceapi.dll',
    'portabledeviceclassextension.dll',
    'portabledeviceconnectapi.dll',
    'portabledevicestatus.dll',
    'portabledevicesyncprovider.dll',
    'portabledevicetypes.dll',
    'portabledevicewiacompat.dll',
    'portabledevicewmdrm.dll',
    'pots.dll',
    'powercpl.dll',
    'powrprof.dll',
    'ppcsnap.dll',
    'presentationcffrasterizernative_v0300.dll',
    'presentationhostproxy.dll',
    'presentationnative_v0300.dll',
    'prflbmsg.dll',
    'printui.dll',
    'prncache.dll',
    'prnfldr.dll',
    'prnntfy.dll',
    'prntvpt.dll',
    'profapi.dll',
    'propsys.dll',
    'provsvc.dll',
    'provthrd.dll',
    'psapi.dll',
    'psbase.dll',
    'pshed.dll',
    'psisdecd.dll',
    'pstorec.dll',
    'pstorsvc.dll',
    'puiapi.dll',
    'puiobj.dll',
    'pwrshplugin.dll',
    'qagent.dll',
    'qasf.dll',
    'qcap.dll',
    'qcliprov.dll',
    'qdv.dll',
    'qdvd.dll',
    'qedit.dll',
    'qedwipes.dll',
    'qmgrprxy.dll',
    'qshvhost.dll',
    'qsvrmgmt.dll',
    'quartz.dll',
    'query.dll',
    'qutil.dll',
    'qwave.dll',
    'racengn.dll',
    'racpldlg.dll',
    'radardt.dll',
    'radarrs.dll',
    'rasadhlp.dll',
    'rasapi32.dll',
    'rascfg.dll',
    'raschap.dll',
    'rasctrs.dll',
    'rasdiag.dll',
    'rasdlg.dll',
    'rasgcw.dll',
    'rasman.dll',
    'rasmm.dll',
    'rasmontr.dll',
    'rasmxs.dll',
    'rasplap.dll',
    'rasppp.dll',
    'rasser.dll',
    'rastapi.dll',
    'rastls.dll',
    'rdpcore.dll',
    'rdpd3d.dll',
    'rdpencom.dll',
    'rdpendp.dll',
    'rdprefdrvapi.dll',
    'rdvgumd32.dll',
    'reagent.dll',
    'regapi.dll',
    'regctrl.dll',
    'remotepg.dll',
    'resampledmo.dll',
    'resutils.dll',
    'rgb9rast.dll',
    'riched20.dll',
    'riched32.dll',
    'rnr20.dll',
    'rpcdiag.dll',
    'rpchttp.dll',
    'rpcndfp.dll',
    'rpcns4.dll',
    'rpcnsh.dll',
    'rpcrt4.dll',
    'rpcrtremote.dll',
    'rsaenh.dll',
    'rshx32.dll',
    'rstrtmgr.dll',
    'rtffilt.dll',
    'rtm.dll',
    'rtutils.dll',
    'samcli.dll',
    'samlib.dll',
    'sampleres.dll',
    'sas.dll',
    'sbe.dll',
    'sbeio.dll',
    'sberes.dll',
    'scansetting.dll',
    'scarddlg.dll',
    'scecli.dll',
    'scesrv.dll',
    'schannel.dll',
    'schedcli.dll',
    'scksp.dll',
    'scripto.dll',
    'scrobj.dll',
    'scrptadm.dll',
    'scrrun.dll',
    'sdiageng.dll',
    'sdiagprv.dll',
    'sdohlp.dll',
    'searchfolder.dll',
    'sechost.dll',
    'secproc.dll',
    'secproc_isv.dll',
    'secproc_ssp.dll',
    'secproc_ssp_isv.dll',
    'secur32.dll',
    'security.dll',
    'sendmail.dll',
    'sens.dll',
    'sensapi.dll',
    'sensorsapi.dll',
    'sensorscpl.dll',
    'serialui.dll',
    'serwvdrv.dll',
    'sessenv.dll',
    'setupapi.dll',
    'setupcln.dll',
    'sfc.dll',
    'sfc_os.dll',
    'shacct.dll',
    'shdocvw.dll',
    'shell32.dll',
    'shellstyle.dll',
    'shfolder.dll',
    'shgina.dll',
    'shimeng.dll',
    'shimgvw.dll',
    'shlwapi.dll',
    'shpafact.dll',
    'shsetup.dll',
    'shsvcs.dll',
    'shunimpl.dll',
    'shwebsvc.dll',
    'signdrv.dll',
    'sisbkup.dll',
    'slc.dll',
    'slcext.dll',
    'slwga.dll',
    'smartcardcredentialprovider.dll',
    'smbhelperclass.dll',
    'sndvolsso.dll',
    'snmpapi.dll',
    'softkbd.dll',
    'softpub.dll',
    'sortserver2003compat.dll',
    'sortwindows6compat.dll',
    'spbcd.dll',
    'spfileq.dll',
    'spinf.dll',
    'spnet.dll',
    'spopk.dll',
    'spp.dll',
    'sppc.dll',
    'sppcc.dll',
    'sppcext.dll',
    'sppcomapi.dll',
    'sppcommdlg.dll',
    'sppinst.dll',
    'sppwmi.dll',
    'spwinsat.dll',
    'spwizeng.dll',
    'spwizimg.dll',
    'spwizres.dll',
    'spwmp.dll',
    'sqlceoledb30.dll',
    'sqlceqp30.dll',
    'sqlcese30.dll',
    'sqlsrv32.dll',
    'sqlunirl.dll',
    'sqlwid.dll',
    'sqlwoa.dll',
    'sqmapi.dll',
    'srchadmin.dll',
    'srclient.dll',
    'srhelper.dll',
    'srpuxnativesnapin.dll',
    'srvcli.dll',
    'sscore.dll',
    'ssdpapi.dll',
    'sspicli.dll',
    'ssshim.dll',
    'stclient.dll',
    'sti.dll',
    'stobject.dll',
    'storage.dll',
    'storagecontexthandler.dll',
    'storprop.dll',
    'structuredquery.dll',
    'sud.dll',
    'sxproxy.dll',
    'sxs.dll',
    'sxshared.dll',
    'sxsstore.dll',
    'synccenter.dll',
    'synceng.dll',
    'synchostps.dll',
    'syncinfrastructure.dll',
    'syncinfrastructureps.dll',
    'syncreg.dll',
    'syncui.dll',
    'syssetup.dll',
    'systemcpl.dll',
    't2embed.dll',
    'tapi3.dll',
    'tapi32.dll',
    'tapimigplugin.dll',
    'tapiperf.dll',
    'tapisrv.dll',
    'tapisysprep.dll',
    'tapiui.dll',
    'taskcomp.dll',
    'taskschd.dll',
    'taskschdps.dll',
    'tbs.dll',
    'tcpipcfg.dll',
    'tcpmonui.dll',
    'tdh.dll',
    'termmgr.dll',
    'thawbrkr.dll',
    'themecpl.dll',
    'themeui.dll',
    'thumbcache.dll',
    'timedatemuicallback.dll',
    'tlscsp.dll',
    'tpmcompc.dll',
    'tquery.dll',
    'traffic.dll',
    'trapi.dll',
    'tsbyuv.dll',
    'tschannel.dll',
    'tsgqec.dll',
    'tsmf.dll',
    'tspkg.dll',
    'tsworkspace.dll',
    'tvratings.dll',
    'twext.dll',
    'txflog.dll',
    'txfw32.dll',
    'typelib.dll',
    'tzres.dll',
    'ubpm.dll',
    'ucmhc.dll',
    'ucrtbase.dll',
    'udhisapi.dll',
    'uexfat.dll',
    'ufat.dll',
    'uianimation.dll',
    'uiautomationcore.dll',
    'uicom.dll',
    'uiribbon.dll',
    'uiribbonres.dll',
    'ulib.dll',
    'umdmxfrm.dll',
    'unimdmat.dll',
    'uniplat.dll',
    'untfs.dll',
    'upnp.dll',
    'upnphost.dll',
    'ureg.dll',
    'url.dll',
    'urlmon.dll',
    'usbceip.dll',
    'usbperf.dll',
    'usbui.dll',
    'user32.dll',
    'useraccountcontrolsettings.dll',
    'usercpl.dll',
    'userenv.dll',
    'usp10.dll',
    'utildll.dll',
    'uudf.dll',
    'uxinit.dll',
    'uxlib.dll',
    'uxlibres.dll',
    'uxtheme.dll',
    'van.dll',
    'vault.dll',
    'vaultcli.dll',
    'vbajet32.dll',
    'vbscript.dll',
    'vdmdbg.dll',
    'vdsbas.dll',
    'vdsdyn.dll',
    'vdsvd.dll',
    'vds_ps.dll',
    'verifier.dll',
    'version.dll',
    'vfpodbc.dll',
    'vfwwdm32.dll',
    'vidreszr.dll',
    'virtdisk.dll',
    'vpnikeapi.dll',
    'vssapi.dll',
    'vsstrace.dll',
    'vss_ps.dll',
    'w32topl.dll',
    'wabsyncprovider.dll',
    'wavemsp.dll',
    'wbemcomn.dll',
    'wcnapi.dll',
    'wcncsvc.dll',
    'wcneapauthproxy.dll',
    'wcneappeerproxy.dll',
    'wcnwiz.dll',
    'wcspluginservice.dll',
    'wdc.dll',
    'wdi.dll',
    'wdigest.dll',
    'wdscore.dll',
    'webcheck.dll',
    'webclnt.dll',
    'webio.dll',
    'webservices.dll',
    'wecapi.dll',
    'wer.dll',
    'werdiagcontroller.dll',
    'werui.dll',
    'wevtapi.dll',
    'wevtfwd.dll',
    'wfapigp.dll',
    'wfhc.dll',
    'whealogr.dll',
    'whhelper.dll',
    'wiaaut.dll',
    'wiadefui.dll',
    'wiadss.dll',
    'wiaextensionhost64.dll',
    'wiascanprofiles.dll',
    'wiashext.dll',
    'wiatrace.dll',
    'wiavideo.dll',
    'wimgapi.dll',
    'win32spl.dll',
    'winbio.dll',
    'winbrand.dll',
    'wincredprovider.dll',
    'windowscodecs.dll',
    'windowscodecsext.dll',
    'winfax.dll',
    'winhttp.dll',
    'wininet.dll',
    'winipsec.dll',
    'winmm.dll',
    'winnsi.dll',
    'winrnr.dll',
    'winrscmd.dll',
    'winrsmgr.dll',
    'winrssrv.dll',
    'winsatapi.dll',
    'winscard.dll',
    'winshfhc.dll',
    'winsockhc.dll',
    'winsrpc.dll',
    'winsta.dll',
    'winsync.dll',
    'winsyncmetastore.dll',
    'winsyncproviders.dll',
    'wintrust.dll',
    'winusb.dll',
    'wkscli.dll',
    'wksprtps.dll',
    'wlanapi.dll',
    'wlancfg.dll',
    'wlanconn.dll',
    'wlandlg.dll',
    'wlangpui.dll',
    'wlanhlp.dll',
    'wlaninst.dll',
    'wlanmm.dll',
    'wlanmsm.dll',
    'wlanpref.dll',
    'wlansec.dll',
    'wlanui.dll',
    'wlanutil.dll',
    'wldap32.dll',
    'wlgpclnt.dll',
    'wls0wndh.dll',
    'wmadmod.dll',
    'wmadmoe.dll',
    'wmasf.dll',
    'wmcodecdspps.dll',
    'wmdmlog.dll',
    'wmdmps.dll',
    'wmdrmdev.dll',
    'wmdrmnet.dll',
    'wmdrmsdk.dll',
    'wmerror.dll',
    'wmi.dll',
    'wmidx.dll',
    'wmiprop.dll',
    'wmnetmgr.dll',
    'wmp.dll',
    'wmpcm.dll',
    'wmpdui.dll',
    'wmpdxm.dll',
    'wmpeffects.dll',
    'wmpencen.dll',
    'wmphoto.dll',
    'wmploc.dll',
    'wmpmde.dll',
    'wmpps.dll',
    'wmpshell.dll',
    'wmpsrcwp.dll',
    'wmsgapi.dll',
    'wmspdmod.dll',
    'wmspdmoe.dll',
    'wmvcore.dll',
    'wmvdecod.dll',
    'wmvdspa.dll',
    'wmvencod.dll',
    'wmvsdecd.dll',
    'wmvsencd.dll',
    'wmvxencd.dll',
    'wow32.dll',
    'wpc.dll',
    'wpcao.dll',
    'wpcsvc.dll',
    'wpdshext.dll',
    'wpdshserviceobj.dll',
    'wpdsp.dll',
    'wpdwcn.dll',
    'ws2help.dll',
    'ws2_32.dll',
    'wscapi.dll',
    'wscinterop.dll',
    'wscisvif.dll',
    'wscmisetup.dll',
    'wscproxystub.dll',
    'wsdapi.dll',
    'wsdchngr.dll',
    'wsecedit.dll',
    'wshbth.dll',
    'wshcon.dll',
    'wshelper.dll',
    'wshext.dll',
    'wship6.dll',
    'wshirda.dll',
    'wshqos.dll',
    'wshrm.dll',
    'wshtcpip.dll',
    'wsmanmigrationplugin.dll',
    'wsmauto.dll',
    'wsmplpxy.dll',
    'wsmres.dll',
    'wsmsvc.dll',
    'wsmwmipl.dll',
    'wsnmp32.dll',
    'wsock32.dll',
    'wtsapi32.dll',
    'wuapi.dll',
    'wudriver.dll',
    'wups.dll',
    'wuwebv.dll',
    'wvc.dll',
    'wwanapi.dll',
    'wwapi.dll',
    'wzcdlg.dll',
    'xinput9_1_0.dll',
    'xmlfilter.dll',
    'xmllite.dll',
    'xmlprovi.dll',
    'xolehlp.dll',
    'xpsfilt.dll',
    'xpsgdiconverter.dll',
    'xpsprint.dll',
    'xpsrasterservice.dll',
    'xpsservices.dll',
    'xpsshhdr.dll',
    'xpssvcs.dll',
    'xwizards.dll',
    'xwreg.dll',
    'xwtpdui.dll',
    'xwtpw32.dll',
    'zipfldr.dll',
    'msacm32.drv',
    'wdmaud.drv',
    'winspool.drv',
}

# ignore_names_64 is a set containing the lowercase names of all DLLs that can
# be assumed to be present on 64-bit Windows 7 or later. The set is generated
# from the file System32.txt, which is a list of all files with extension .dll
# or .drv found in C:\Windows\System32 on a vanilla Windows 7 Ultimate SP1
# 64-bit installation.
ignore_names_64 = {
    'aaclient.dll',
    'accessibilitycpl.dll',
    'acctres.dll',
    'acledit.dll',
    'aclui.dll',
    'acppage.dll',
    'actioncenter.dll',
    'actioncentercpl.dll',
    'activeds.dll',
    'actxprxy.dll',
    'admtmpl.dll',
    'adprovider.dll',
    'adsldp.dll',
    'adsldpc.dll',
    'adsmsext.dll',
    'adsnt.dll',
    'adtschema.dll',
    'advapi32.dll',
    'advpack.dll',
    'aecache.dll',
    'aeevts.dll',
    'alttab.dll',
    'amstream.dll',
    'amxread.dll',
    'apds.dll',
    'api-ms-win-core-file-l1-2-0.dll',
    'api-ms-win-core-file-l2-1-0.dll',
    'api-ms-win-core-localization-l1-2-0.dll',
    'api-ms-win-core-processthreads-l1-1-1.dll',
    'api-ms-win-core-synch-l1-2-0.dll',
    'api-ms-win-core-timezone-l1-1-0.dll',
    'api-ms-win-core-xstate-l2-1-0.dll',
    'api-ms-win-crt-conio-l1-1-0.dll',
    'api-ms-win-crt-convert-l1-1-0.dll',
    'api-ms-win-crt-environment-l1-1-0.dll',
    'api-ms-win-crt-filesystem-l1-1-0.dll',
    'api-ms-win-crt-heap-l1-1-0.dll',
    'api-ms-win-crt-locale-l1-1-0.dll',
    'api-ms-win-crt-math-l1-1-0.dll',
    'api-ms-win-crt-multibyte-l1-1-0.dll',
    'api-ms-win-crt-private-l1-1-0.dll',
    'api-ms-win-crt-process-l1-1-0.dll',
    'api-ms-win-crt-runtime-l1-1-0.dll',
    'api-ms-win-crt-stdio-l1-1-0.dll',
    'api-ms-win-crt-string-l1-1-0.dll',
    'api-ms-win-crt-time-l1-1-0.dll',
    'api-ms-win-crt-utility-l1-1-0.dll',
    'apilogen.dll',
    'apircl.dll',
    'apisetschema.dll',
    'apphelp.dll',
    'apphlpdm.dll',
    'appidapi.dll',
    'appidpolicyengineapi.dll',
    'appmgmts.dll',
    'appmgr.dll',
    'apss.dll',
    'asferror.dll',
    'aspnet_counters.dll',
    'asycfilt.dll',
    'atl.dll',
    'atmfd.dll',
    'atmlib.dll',
    'audiodev.dll',
    'audioeng.dll',
    'audiokse.dll',
    'audioses.dll',
    'auditnativesnapin.dll',
    'auditpolicygpinterop.dll',
    'auditpolmsg.dll',
    'authfwcfg.dll',
    'authfwgp.dll',
    'authfwsnapin.dll',
    'authfwwizfwk.dll',
    'authui.dll',
    'authz.dll',
    'autoplay.dll',
    'auxiliarydisplayapi.dll',
    'auxiliarydisplaycpl.dll',
    'avicap32.dll',
    'avifil32.dll',
    'avrt.dll',
    'azroles.dll',
    'azroleui.dll',
    'azsqlext.dll',
    'basecsp.dll',
    'batmeter.dll',
    'bcrypt.dll',
    'bcryptprimitives.dll',
    'bidispl.dll',
    'biocredprov.dll',
    'bitsperf.dll',
    'bitsprx2.dll',
    'bitsprx3.dll',
    'bitsprx4.dll',
    'bitsprx5.dll',
    'bitsprx6.dll',
    'blackbox.dll',
    'bootvid.dll',
    'browcli.dll',
    'browseui.dll',
    'btpanui.dll',
    'bwcontexthandler.dll',
    'bwunpairelevated.dll',
    'cabinet.dll',
    'cabview.dll',
    'capiprovider.dll',
    'capisp.dll',
    'catsrv.dll',
    'catsrvps.dll',
    'catsrvut.dll',
    'cca.dll',
    'cdosys.dll',
    'certcli.dll',
    'certcredprovider.dll',
    'certenc.dll',
    'certenroll.dll',
    'certenrollui.dll',
    'certmgr.dll',
    'certpoleng.dll',
    'cewmdm.dll',
    'cfgbkend.dll',
    'cfgmgr32.dll',
    'chsbrkr.dll',
    'chtbrkr.dll',
    'chxreadingstringime.dll',
    'cic.dll',
    'clb.dll',
    'clbcatq.dll',
    'clfsw32.dll',
    'cliconfg.dll',
    'clusapi.dll',
    'cmcfg32.dll',
    'cmdial32.dll',
    'cmicryptinstall.dll',
    'cmifw.dll',
    'cmipnpinstall.dll',
    'cmlua.dll',
    'cmpbk32.dll',
    'cmstplua.dll',
    'cmutil.dll',
    'cngaudit.dll',
    'cngprovider.dll',
    'cnvfat.dll',
    'colbact.dll',
    'colorcnv.dll',
    'colorui.dll',
    'comcat.dll',
    'comctl32.dll',
    'comdlg32.dll',
    'compobj.dll',
    'compstui.dll',
    'comrepl.dll',
    'comres.dll',
    'comsnap.dll',
    'comsvcs.dll',
    'comuid.dll',
    'connect.dll',
    'console.dll',
    'cpfilters.dll',
    'credssp.dll',
    'credui.dll',
    'crtdll.dll',
    'crypt32.dll',
    'cryptbase.dll',
    'cryptdlg.dll',
    'cryptdll.dll',
    'cryptext.dll',
    'cryptnet.dll',
    'cryptsp.dll',
    'cryptsvc.dll',
    'cryptui.dll',
    'cryptxml.dll',
    'cscapi.dll',
    'cscdll.dll',
    'cscobj.dll',
    'ctl3d32.dll',
    'c_g18030.dll',
    'c_is2022.dll',
    'c_iscii.dll',
    'd2d1.dll',
    'd3d10.dll',
    'd3d10core.dll',
    'd3d10level9.dll',
    'd3d10warp.dll',
    'd3d10_1.dll',
    'd3d10_1core.dll',
    'd3d11.dll',
    'd3d8.dll',
    'd3d8thk.dll',
    'd3d9.dll',
    'd3dcompiler_47.dll',
    'd3dim.dll',
    'd3dim700.dll',
    'd3dramp.dll',
    'd3dxof.dll',
    'dataclen.dll',
    'davclnt.dll',
    'davhlpr.dll',
    'dbgeng.dll',
    'dbghelp.dll',
    'dbnetlib.dll',
    'dbnmpntw.dll',
    'dciman32.dll',
    'ddaclsys.dll',
    'ddoiproxy.dll',
    'ddores.dll',
    'ddraw.dll',
    'ddrawex.dll',
    'defaultlocationcpl.dll',
    'deskadp.dll',
    'deskmon.dll',
    'deskperf.dll',
    'devenum.dll',
    'devicecenter.dll',
    'devicedisplaystatusmanager.dll',
    'devicemetadataparsers.dll',
    'devicepairing.dll',
    'devicepairingfolder.dll',
    'devicepairinghandler.dll',
    'devicepairingproxy.dll',
    'deviceuxres.dll',
    'devmgr.dll',
    'devobj.dll',
    'devrtl.dll',
    'dfscli.dll',
    'dfshim.dll',
    'dfsshlex.dll',
    'dhcpcmonitor.dll',
    'dhcpcore.dll',
    'dhcpcore6.dll',
    'dhcpcsvc.dll',
    'dhcpcsvc6.dll',
    'dhcpqec.dll',
    'dhcpsapi.dll',
    'difxapi.dll',
    'dimsjob.dll',
    'dimsroam.dll',
    'dinput.dll',
    'dinput8.dll',
    'diskcopy.dll',
    'dispex.dll',
    'display.dll',
    'dmband.dll',
    'dmcompos.dll',
    'dmdlgs.dll',
    'dmdskmgr.dll',
    'dmdskres.dll',
    'dmdskres2.dll',
    'dmime.dll',
    'dmintf.dll',
    'dmloader.dll',
    'dmocx.dll',
    'dmrc.dll',
    'dmscript.dll',
    'dmstyle.dll',
    'dmsynth.dll',
    'dmusic.dll',
    'dmutil.dll',
    'dmvdsitf.dll',
    'dnsapi.dll',
    'dnscmmc.dll',
    'docprop.dll',
    'dot3api.dll',
    'dot3cfg.dll',
    'dot3dlg.dll',
    'dot3gpclnt.dll',
    'dot3gpui.dll',
    'dot3hc.dll',
    'dot3msm.dll',
    'dot3ui.dll',
    'dpapiprovider.dll',
    'dplayx.dll',
    'dpmodemx.dll',
    'dpnaddr.dll',
    'dpnathlp.dll',
    'dpnet.dll',
    'dpnhpast.dll',
    'dpnhupnp.dll',
    'dpnlobby.dll',
    'dpwsockx.dll',
    'dpx.dll',
    'drmmgrtn.dll',
    'drmv2clt.dll',
    'drprov.dll',
    'drt.dll',
    'drtprov.dll',
    'drttransport.dll',
    'drvstore.dll',
    'ds32gt.dll',
    'dsauth.dll',
    'dsdmo.dll',
    'dshowrdpfilter.dll',
    'dskquota.dll',
    'dskquoui.dll',
    'dsound.dll',
    'dsprop.dll',
    'dsquery.dll',
    'dsrole.dll',
    'dssec.dll',
    'dssenh.dll',
    'dsuiext.dll',
    'dswave.dll',
    'dtsh.dll',
    'dui70.dll',
    'duser.dll',
    'dwmapi.dll',
    'dwmcore.dll',
    'dwrite.dll',
    'dxdiagn.dll',
    'dxgi.dll',
    'dxmasf.dll',
    'dxptaskringtone.dll',
    'dxptasksync.dll',
    'dxtmsft.dll',
    'dxtrans.dll',
    'dxva2.dll',
    'eapp3hst.dll',
    'eappcfg.dll',
    'eappgnui.dll',
    'eapphost.dll',
    'eappprxy.dll',
    'eapqec.dll',
    'efsadu.dll',
    'efscore.dll',
    'efsutil.dll',
    'ehstorapi.dll',
    'ehstorpwdmgr.dll',
    'ehstorshell.dll',
    'els.dll',
    'elscore.dll',
    'elshyph.dll',
    'elslad.dll',
    'elstrans.dll',
    'encapi.dll',
    'encdec.dll',
    'eqossnap.dll',
    'es.dll',
    'esent.dll',
    'esentprf.dll',
    'eventcls.dll',
    'evr.dll',
    'explorerframe.dll',
    'expsrv.dll',
    'f3ahvoas.dll',
    'faultrep.dll',
    'fdbth.dll',
    'fdbthproxy.dll',
    'fde.dll',
    'fdeploy.dll',
    'fdpnp.dll',
    'fdproxy.dll',
    'fdssdp.dll',
    'fdwcn.dll',
    'fdwnet.dll',
    'fdwsd.dll',
    'feclient.dll',
    'filemgmt.dll',
    'findnetprinters.dll',
    'firewallapi.dll',
    'firewallcontrolpanel.dll',
    'fltlib.dll',
    'fmifs.dll',
    'fms.dll',
    'fontext.dll',
    'fontsub.dll',
    'fphc.dll',
    'framedyn.dll',
    'framedynos.dll',
    'fthsvc.dll',
    'fundisc.dll',
    'fwcfg.dll',
    'fwpuclnt.dll',
    'fwremotesvr.dll',
    'fxsapi.dll',
    'fxscom.dll',
    'fxscomex.dll',
    'fxsext32.dll',
    'fxsresm.dll',
    'fxsxp32.dll',
    'gameux.dll',
    'gameuxlegacygdfs.dll',
    'gcdef.dll',
    'gdi32.dll',
    'getuname.dll',
    'glmf32.dll',
    'glu32.dll',
    'gpapi.dll',
    'gpedit.dll',
    'gpprefcl.dll',
    'gpprnext.dll',
    'gpscript.dll',
    'gptext.dll',
    'hbaapi.dll',
    'hcproviders.dll',
    'helppaneproxy.dll',
    'hgcpl.dll',
    'hhsetup.dll',
    'hid.dll',
    'hidserv.dll',
    'hlink.dll',
    'hnetcfg.dll',
    'hnetmon.dll',
    'httpapi.dll',
    'htui.dll',
    'ias.dll',
    'iasacct.dll',
    'iasads.dll',
    'iasdatastore.dll',
    'iashlpr.dll',
    'iasmigplugin.dll',
    'iasnap.dll',
    'iaspolcy.dll',
    'iasrad.dll',
    'iasrecst.dll',
    'iassam.dll',
    'iassdo.dll',
    'iassvcs.dll',
    'icardie.dll',
    'icardres.dll',
    'iccvid.dll',
    'icm32.dll',
    'icmp.dll',
    'icmui.dll',
    'iconcodecservice.dll',
    'icsigd.dll',
    'idndl.dll',
    'idstore.dll',
    'ieadvpack.dll',
    'ieapfltr.dll',
    'iedkcs32.dll',
    'ieetwproxystub.dll',
    'ieframe.dll',
    'iepeers.dll',
    'iernonce.dll',
    'iertutil.dll',
    'iesetup.dll',
    'iesysprep.dll',
    'ieui.dll',
    'ifmon.dll',
    'ifsutil.dll',
    'ifsutilx.dll',
    'imagehlp.dll',
    'imageres.dll',
    'imagesp1.dll',
    'imapi.dll',
    'imapi2.dll',
    'imapi2fs.dll',
    'imgutil.dll',
    'imjp10k.dll',
    'imm32.dll',
    'inetcomm.dll',
    'inetmib1.dll',
    'inetres.dll',
    'infocardapi.dll',
    'inked.dll',
    'input.dll',
    'inseng.dll',
    'iologmsg.dll',
    'ipbusenumproxy.dll',
    'iphlpapi.dll',
    'iprop.dll',
    'iprtprio.dll',
    'iprtrmgr.dll',
    'ipsecsnp.dll',
    'ipsmsnap.dll',
    'ir32_32.dll',
    'ir41_qc.dll',
    'ir41_qcx.dll',
    'ir50_32.dll',
    'ir50_qc.dll',
    'ir50_qcx.dll',
    'irclass.dll',
    'iscsicpl.dll',
    'iscsidsc.dll',
    'iscsied.dll',
    'iscsium.dll',
    'iscsiwmi.dll',
    'itircl.dll',
    'itss.dll',
    'itvdata.dll',
    'iyuv_32.dll',
    'javascriptcollectionagent.dll',
    'jscript.dll',
    'jscript9.dll',
    'jscript9diag.dll',
    'jsintl.dll',
    'jsproxy.dll',
    'kbd101.dll',
    'kbd101a.dll',
    'kbd101b.dll',
    'kbd101c.dll',
    'kbd103.dll',
    'kbd106.dll',
    'kbd106n.dll',
    'kbda1.dll',
    'kbda2.dll',
    'kbda3.dll',
    'kbdal.dll',
    'kbdarme.dll',
    'kbdarmw.dll',
    'kbdax2.dll',
    'kbdaze.dll',
    'kbdazel.dll',
    'kbdbash.dll',
    'kbdbe.dll',
    'kbdbene.dll',
    'kbdbgph.dll',
    'kbdbgph1.dll',
    'kbdbhc.dll',
    'kbdblr.dll',
    'kbdbr.dll',
    'kbdbu.dll',
    'kbdbulg.dll',
    'kbdca.dll',
    'kbdcan.dll',
    'kbdcr.dll',
    'kbdcz.dll',
    'kbdcz1.dll',
    'kbdcz2.dll',
    'kbdda.dll',
    'kbddiv1.dll',
    'kbddiv2.dll',
    'kbddv.dll',
    'kbdes.dll',
    'kbdest.dll',
    'kbdfa.dll',
    'kbdfc.dll',
    'kbdfi.dll',
    'kbdfi1.dll',
    'kbdfo.dll',
    'kbdfr.dll',
    'kbdgae.dll',
    'kbdgeo.dll',
    'kbdgeoer.dll',
    'kbdgeoqw.dll',
    'kbdgkl.dll',
    'kbdgr.dll',
    'kbdgr1.dll',
    'kbdgrlnd.dll',
    'kbdhau.dll',
    'kbdhe.dll',
    'kbdhe220.dll',
    'kbdhe319.dll',
    'kbdheb.dll',
    'kbdhela2.dll',
    'kbdhela3.dll',
    'kbdhept.dll',
    'kbdhu.dll',
    'kbdhu1.dll',
    'kbdibm02.dll',
    'kbdibo.dll',
    'kbdic.dll',
    'kbdinasa.dll',
    'kbdinbe1.dll',
    'kbdinbe2.dll',
    'kbdinben.dll',
    'kbdindev.dll',
    'kbdinguj.dll',
    'kbdinhin.dll',
    'kbdinkan.dll',
    'kbdinmal.dll',
    'kbdinmar.dll',
    'kbdinori.dll',
    'kbdinpun.dll',
    'kbdintam.dll',
    'kbdintel.dll',
    'kbdinuk2.dll',
    'kbdir.dll',
    'kbdit.dll',
    'kbdit142.dll',
    'kbdiulat.dll',
    'kbdjpn.dll',
    'kbdkaz.dll',
    'kbdkhmr.dll',
    'kbdkor.dll',
    'kbdkyr.dll',
    'kbdla.dll',
    'kbdlao.dll',
    'kbdlk41a.dll',
    'kbdlt.dll',
    'kbdlt1.dll',
    'kbdlt2.dll',
    'kbdlv.dll',
    'kbdlv1.dll',
    'kbdmac.dll',
    'kbdmacst.dll',
    'kbdmaori.dll',
    'kbdmlt47.dll',
    'kbdmlt48.dll',
    'kbdmon.dll',
    'kbdmonmo.dll',
    'kbdne.dll',
    'kbdnec.dll',
    'kbdnec95.dll',
    'kbdnecat.dll',
    'kbdnecnt.dll',
    'kbdnepr.dll',
    'kbdno.dll',
    'kbdno1.dll',
    'kbdnso.dll',
    'kbdpash.dll',
    'kbdpl.dll',
    'kbdpl1.dll',
    'kbdpo.dll',
    'kbdro.dll',
    'kbdropr.dll',
    'kbdrost.dll',
    'kbdru.dll',
    'kbdru1.dll',
    'kbdsf.dll',
    'kbdsg.dll',
    'kbdsl.dll',
    'kbdsl1.dll',
    'kbdsmsfi.dll',
    'kbdsmsno.dll',
    'kbdsn1.dll',
    'kbdsorex.dll',
    'kbdsors1.dll',
    'kbdsorst.dll',
    'kbdsp.dll',
    'kbdsw.dll',
    'kbdsw09.dll',
    'kbdsyr1.dll',
    'kbdsyr2.dll',
    'kbdtajik.dll',
    'kbdtat.dll',
    'kbdth0.dll',
    'kbdth1.dll',
    'kbdth2.dll',
    'kbdth3.dll',
    'kbdtiprc.dll',
    'kbdtuf.dll',
    'kbdtuq.dll',
    'kbdturme.dll',
    'kbdughr.dll',
    'kbdughr1.dll',
    'kbduk.dll',
    'kbdukx.dll',
    'kbdur.dll',
    'kbdur1.dll',
    'kbdurdu.dll',
    'kbdus.dll',
    'kbdusa.dll',
    'kbdusl.dll',
    'kbdusr.dll',
    'kbdusx.dll',
    'kbduzb.dll',
    'kbdvntc.dll',
    'kbdwol.dll',
    'kbdyak.dll',
    'kbdyba.dll',
    'kbdycc.dll',
    'kbdycl.dll',
    'kerberos.dll',
    'kernel32.dll',
    'kernelbase.dll',
    'keyiso.dll',
    'keymgr.dll',
    'korwbrkr.dll',
    'ksuser.dll',
    'ktmw32.dll',
    'l2gpstore.dll',
    'l2nacp.dll',
    'l2sechc.dll',
    'laprxy.dll',
    'licmgr10.dll',
    'linkinfo.dll',
    'loadperf.dll',
    'localsec.dll',
    'locationapi.dll',
    'loghours.dll',
    'logoncli.dll',
    'lpk.dll',
    'lsmproxy.dll',
    'luainstall.dll',
    'lz32.dll',
    'magnification.dll',
    'mapi32.dll',
    'mapistub.dll',
    'mcewmdrmndbootstrap.dll',
    'mciavi32.dll',
    'mcicda.dll',
    'mciqtz32.dll',
    'mciseq.dll',
    'mciwave.dll',
    'mctres.dll',
    'mdminst.dll',
    'mediametadatahandler.dll',
    'mf.dll',
    'mf3216.dll',
    'mfaacenc.dll',
    'mfc40.dll',
    'mfc40u.dll',
    'mfc42.dll',
    'mfc42u.dll',
    'mfcsubs.dll',
    'mfds.dll',
    'mfdvdec.dll',
    'mferror.dll',
    'mfh264enc.dll',
    'mfmjpegdec.dll',
    'mfplat.dll',
    'mfplay.dll',
    'mfps.dll',
    'mfreadwrite.dll',
    'mfvdsp.dll',
    'mfwmaaec.dll',
    'mgmtapi.dll',
    'midimap.dll',
    'migisol.dll',
    'miguiresource.dll',
    'mimefilt.dll',
    'mlang.dll',
    'mmcbase.dll',
    'mmci.dll',
    'mmcico.dll',
    'mmcndmgr.dll',
    'mmcshext.dll',
    'mmdevapi.dll',
    'mmres.dll',
    'modemui.dll',
    'moricons.dll',
    'mp3dmod.dll',
    'mp43decd.dll',
    'mp4sdecd.dll',
    'mpg4decd.dll',
    'mpr.dll',
    'mprapi.dll',
    'mprddm.dll',
    'mprdim.dll',
    'mprmsg.dll',
    'msaatext.dll',
    'msac3enc.dll',
    'msacm32.dll',
    'msafd.dll',
    'msasn1.dll',
    'msaudite.dll',
    'mscandui.dll',
    'mscat32.dll',
    'msclmd.dll',
    'mscms.dll',
    'mscoree.dll',
    'mscorier.dll',
    'mscories.dll',
    'mscpx32r.dll',
    'mscpxl32.dll',
    'msctf.dll',
    'msctfmonitor.dll',
    'msctfp.dll',
    'msctfui.dll',
    'msdadiag.dll',
    'msdart.dll',
    'msdelta.dll',
    'msdmo.dll',
    'msdrm.dll',
    'msdtcprx.dll',
    'msdtcuiu.dll',
    'msdtcvsp1res.dll',
    'msexch40.dll',
    'msexcl40.dll',
    'msfeeds.dll',
    'msfeedsbs.dll',
    'msftedit.dll',
    'mshtml.dll',
    'mshtmldac.dll',
    'mshtmled.dll',
    'mshtmler.dll',
    'mshtmlmedia.dll',
    'msi.dll',
    'msidcrl30.dll',
    'msident.dll',
    'msidle.dll',
    'msidntld.dll',
    'msieftp.dll',
    'msihnd.dll',
    'msiltcfg.dll',
    'msimg32.dll',
    'msimsg.dll',
    'msimtf.dll',
    'msisip.dll',
    'msjet40.dll',
    'msjetoledb40.dll',
    'msjint40.dll',
    'msjter40.dll',
    'msjtes40.dll',
    'msls31.dll',
    'msltus40.dll',
    'msmpeg2adec.dll',
    'msmpeg2enc.dll',
    'msmpeg2vdec.dll',
    'msnetobj.dll',
    'msobjs.dll',
    'msoeacct.dll',
    'msoert2.dll',
    'msorc32r.dll',
    'msorcl32.dll',
    'mspatcha.dll',
    'mspbde40.dll',
    'msports.dll',
    'msrating.dll',
    'msrd2x40.dll',
    'msrd3x40.dll',
    'msrdc.dll',
    'msrdpwebaccess.dll',
    'msrepl40.dll',
    'msrle32.dll',
    'msscntrs.dll',
    'msscp.dll',
    'mssha.dll',
    'msshavmsg.dll',
    'msshooks.dll',
    'mssign32.dll',
    'mssip32.dll',
    'mssitlb.dll',
    'mssph.dll',
    'mssphtb.dll',
    'mssprxy.dll',
    'mssrch.dll',
    'mssvp.dll',
    'msswch.dll',
    'mstask.dll',
    'mstext40.dll',
    'mstscax.dll',
    'msutb.dll',
    'msv1_0.dll',
    'msvbvm60.dll',
    'msvcirt.dll',
    'msvcp110_clr0400.dll',
    'msvcp120_clr0400.dll',
    'msvcp60.dll',
    'msvcr100_clr0400.dll',
    'msvcr110_clr0400.dll',
    'msvcr120_clr0400.dll',
    'msvcrt.dll',
    'msvcrt20.dll',
    'msvcrt40.dll',
    'msvfw32.dll',
    'msvidc32.dll',
    'msvidctl.dll',
    'mswdat10.dll',
    'mswmdm.dll',
    'mswsock.dll',
    'mswstr10.dll',
    'msxbde40.dll',
    'msxml3.dll',
    'msxml3r.dll',
    'msxml6.dll',
    'msxml6r.dll',
    'msyuv.dll',
    'mtxclu.dll',
    'mtxdm.dll',
    'mtxex.dll',
    'mtxlegih.dll',
    'mtxoci.dll',
    'muifontsetup.dll',
    'mycomput.dll',
    'mydocs.dll',
    'napcrypt.dll',
    'napdsnap.dll',
    'naphlpr.dll',
    'napinsp.dll',
    'napipsec.dll',
    'napmontr.dll',
    'nativehooks.dll',
    'naturallanguage6.dll',
    'ncdprop.dll',
    'nci.dll',
    'ncobjapi.dll',
    'ncrypt.dll',
    'ncryptui.dll',
    'ncsi.dll',
    'nddeapi.dll',
    'ndfapi.dll',
    'ndfetw.dll',
    'ndfhcdiscovery.dll',
    'ndiscapcfg.dll',
    'ndishc.dll',
    'ndproxystub.dll',
    'negoexts.dll',
    'netapi32.dll',
    'netbios.dll',
    'netcenter.dll',
    'netcfgx.dll',
    'netcorehc.dll',
    'netdiagfx.dll',
    'netevent.dll',
    'netfxperf.dll',
    'neth.dll',
    'netid.dll',
    'netiohlp.dll',
    'netjoin.dll',
    'netlogon.dll',
    'netmsg.dll',
    'netplwiz.dll',
    'netprof.dll',
    'netprofm.dll',
    'netshell.dll',
    'netutils.dll',
    'networkexplorer.dll',
    'networkitemfactory.dll',
    'networkmap.dll',
    'newdev.dll',
    'nlaapi.dll',
    'nlhtml.dll',
    'nlmgp.dll',
    'nlmsprep.dll',
    'nlsbres.dll',
    'nlsdata0000.dll',
    'nlsdata0001.dll',
    'nlsdata0002.dll',
    'nlsdata0003.dll',
    'nlsdata0007.dll',
    'nlsdata0009.dll',
    'nlsdata000a.dll',
    'nlsdata000c.dll',
    'nlsdata000d.dll',
    'nlsdata000f.dll',
    'nlsdata0010.dll',
    'nlsdata0011.dll',
    'nlsdata0013.dll',
    'nlsdata0018.dll',
    'nlsdata0019.dll',
    'nlsdata001a.dll',
    'nlsdata001b.dll',
    'nlsdata001d.dll',
    'nlsdata0020.dll',
    'nlsdata0021.dll',
    'nlsdata0022.dll',
    'nlsdata0024.dll',
    'nlsdata0026.dll',
    'nlsdata0027.dll',
    'nlsdata002a.dll',
    'nlsdata0039.dll',
    'nlsdata003e.dll',
    'nlsdata0045.dll',
    'nlsdata0046.dll',
    'nlsdata0047.dll',
    'nlsdata0049.dll',
    'nlsdata004a.dll',
    'nlsdata004b.dll',
    'nlsdata004c.dll',
    'nlsdata004e.dll',
    'nlsdata0414.dll',
    'nlsdata0416.dll',
    'nlsdata0816.dll',
    'nlsdata081a.dll',
    'nlsdata0c1a.dll',
    'nlsdl.dll',
    'nlslexicons0001.dll',
    'nlslexicons0002.dll',
    'nlslexicons0003.dll',
    'nlslexicons0007.dll',
    'nlslexicons0009.dll',
    'nlslexicons000a.dll',
    'nlslexicons000c.dll',
    'nlslexicons000d.dll',
    'nlslexicons000f.dll',
    'nlslexicons0010.dll',
    'nlslexicons0011.dll',
    'nlslexicons0013.dll',
    'nlslexicons0018.dll',
    'nlslexicons0019.dll',
    'nlslexicons001a.dll',
    'nlslexicons001b.dll',
    'nlslexicons001d.dll',
    'nlslexicons0020.dll',
    'nlslexicons0021.dll',
    'nlslexicons0022.dll',
    'nlslexicons0024.dll',
    'nlslexicons0026.dll',
    'nlslexicons0027.dll',
    'nlslexicons002a.dll',
    'nlslexicons0039.dll',
    'nlslexicons003e.dll',
    'nlslexicons0045.dll',
    'nlslexicons0046.dll',
    'nlslexicons0047.dll',
    'nlslexicons0049.dll',
    'nlslexicons004a.dll',
    'nlslexicons004b.dll',
    'nlslexicons004c.dll',
    'nlslexicons004e.dll',
    'nlslexicons0414.dll',
    'nlslexicons0416.dll',
    'nlslexicons0816.dll',
    'nlslexicons081a.dll',
    'nlslexicons0c1a.dll',
    'nlsmodels0011.dll',
    'normaliz.dll',
    'npmproxy.dll',
    'nshhttp.dll',
    'nshipsec.dll',
    'nshwfp.dll',
    'nsi.dll',
    'ntdll.dll',
    'ntdsapi.dll',
    'ntlanman.dll',
    'ntlanui2.dll',
    'ntmarta.dll',
    'ntprint.dll',
    'ntshrui.dll',
    'ntvdm64.dll',
    'objsel.dll',
    'occache.dll',
    'ocsetapi.dll',
    'odbc32.dll',
    'odbc32gt.dll',
    'odbcbcp.dll',
    'odbcconf.dll',
    'odbccp32.dll',
    'odbccr32.dll',
    'odbccu32.dll',
    'odbcint.dll',
    'odbcji32.dll',
    'odbcjt32.dll',
    'odbctrac.dll',
    'oddbse32.dll',
    'odexl32.dll',
    'odfox32.dll',
    'odpdx32.dll',
    'odtext32.dll',
    'offfilt.dll',
    'ogldrv.dll',
    'ole2.dll',
    'ole2disp.dll',
    'ole2nls.dll',
    'ole32.dll',
    'oleacc.dll',
    'oleacchooks.dll',
    'oleaccrc.dll',
    'oleaut32.dll',
    'olecli32.dll',
    'oledlg.dll',
    'oleprn.dll',
    'olepro32.dll',
    'oleres.dll',
    'olesvr32.dll',
    'olethk32.dll',
    'onex.dll',
    'onexui.dll',
    'onlineidcpl.dll',
    'oobefldr.dll',
    'opcservices.dll',
    'opengl32.dll',
    'osbaseln.dll',
    'osuninst.dll',
    'p2p.dll',
    'p2pcollab.dll',
    'p2pgraph.dll',
    'p2pnetsh.dll',
    'packager.dll',
    'panmap.dll',
    'pautoenr.dll',
    'pcaui.dll',
    'pcwum.dll',
    'pdh.dll',
    'pdhui.dll',
    'peerdist.dll',
    'peerdistsh.dll',
    'perfcentercpl.dll',
    'perfctrs.dll',
    'perfdisk.dll',
    'perfnet.dll',
    'perfos.dll',
    'perfproc.dll',
    'perfts.dll',
    'photometadatahandler.dll',
    'photowiz.dll',
    'pid.dll',
    'pidgenx.dll',
    'pifmgr.dll',
    'pku2u.dll',
    'pla.dll',
    'playsndsrv.dll',
    'pmcsnap.dll',
    'pngfilt.dll',
    'pnidui.dll',
    'pnpsetup.dll',
    'pnrpnsp.dll',
    'polstore.dll',
    'portabledeviceapi.dll',
    'portabledeviceclassextension.dll',
    'portabledeviceconnectapi.dll',
    'portabledevicestatus.dll',
    'portabledevicesyncprovider.dll',
    'portabledevicetypes.dll',
    'portabledevicewiacompat.dll',
    'portabledevicewmdrm.dll',
    'pots.dll',
    'powercpl.dll',
    'powrprof.dll',
    'ppcsnap.dll',
    'presentationcffrasterizernative_v0300.dll',
    'presentationhostproxy.dll',
    'presentationnative_v0300.dll',
    'prflbmsg.dll',
    'printui.dll',
    'prncache.dll',
    'prnfldr.dll',
    'prnntfy.dll',
    'prntvpt.dll',
    'profapi.dll',
    'propsys.dll',
    'provsvc.dll',
    'provthrd.dll',
    'psapi.dll',
    'psbase.dll',
    'pshed.dll',
    'psisdecd.dll',
    'pstorec.dll',
    'pstorsvc.dll',
    'puiapi.dll',
    'puiobj.dll',
    'pwrshplugin.dll',
    'qagent.dll',
    'qasf.dll',
    'qcap.dll',
    'qcliprov.dll',
    'qdv.dll',
    'qdvd.dll',
    'qedit.dll',
    'qedwipes.dll',
    'qmgrprxy.dll',
    'qshvhost.dll',
    'qsvrmgmt.dll',
    'quartz.dll',
    'query.dll',
    'qutil.dll',
    'qwave.dll',
    'racengn.dll',
    'racpldlg.dll',
    'radardt.dll',
    'radarrs.dll',
    'rasadhlp.dll',
    'rasapi32.dll',
    'rascfg.dll',
    'raschap.dll',
    'rasctrs.dll',
    'rasdiag.dll',
    'rasdlg.dll',
    'rasgcw.dll',
    'rasman.dll',
    'rasmm.dll',
    'rasmontr.dll',
    'rasmxs.dll',
    'rasplap.dll',
    'rasppp.dll',
    'rasser.dll',
    'rastapi.dll',
    'rastls.dll',
    'rdpcore.dll',
    'rdpd3d.dll',
    'rdpencom.dll',
    'rdpendp.dll',
    'rdprefdrvapi.dll',
    'rdvgumd32.dll',
    'reagent.dll',
    'regapi.dll',
    'regctrl.dll',
    'remotepg.dll',
    'resampledmo.dll',
    'resutils.dll',
    'rgb9rast.dll',
    'riched20.dll',
    'riched32.dll',
    'rnr20.dll',
    'rpcdiag.dll',
    'rpchttp.dll',
    'rpcndfp.dll',
    'rpcns4.dll',
    'rpcnsh.dll',
    'rpcrt4.dll',
    'rpcrtremote.dll',
    'rsaenh.dll',
    'rshx32.dll',
    'rstrtmgr.dll',
    'rtffilt.dll',
    'rtm.dll',
    'rtutils.dll',
    'samcli.dll',
    'samlib.dll',
    'sampleres.dll',
    'sas.dll',
    'sbe.dll',
    'sbeio.dll',
    'sberes.dll',
    'scansetting.dll',
    'scarddlg.dll',
    'scecli.dll',
    'scesrv.dll',
    'schannel.dll',
    'schedcli.dll',
    'scksp.dll',
    'scripto.dll',
    'scrobj.dll',
    'scrptadm.dll',
    'scrrun.dll',
    'sdiageng.dll',
    'sdiagprv.dll',
    'sdohlp.dll',
    'searchfolder.dll',
    'sechost.dll',
    'secproc.dll',
    'secproc_isv.dll',
    'secproc_ssp.dll',
    'secproc_ssp_isv.dll',
    'secur32.dll',
    'security.dll',
    'sendmail.dll',
    'sens.dll',
    'sensapi.dll',
    'sensorsapi.dll',
    'sensorscpl.dll',
    'serialui.dll',
    'serwvdrv.dll',
    'sessenv.dll',
    'setupapi.dll',
    'setupcln.dll',
    'sfc.dll',
    'sfc_os.dll',
    'shacct.dll',
    'shdocvw.dll',
    'shell32.dll',
    'shellstyle.dll',
    'shfolder.dll',
    'shgina.dll',
    'shimeng.dll',
    'shimgvw.dll',
    'shlwapi.dll',
    'shpafact.dll',
    'shsetup.dll',
    'shsvcs.dll',
    'shunimpl.dll',
    'shwebsvc.dll',
    'signdrv.dll',
    'sisbkup.dll',
    'slc.dll',
    'slcext.dll',
    'slwga.dll',
    'smartcardcredentialprovider.dll',
    'smbhelperclass.dll',
    'sndvolsso.dll',
    'snmpapi.dll',
    'softkbd.dll',
    'softpub.dll',
    'sortserver2003compat.dll',
    'sortwindows6compat.dll',
    'spbcd.dll',
    'spfileq.dll',
    'spinf.dll',
    'spnet.dll',
    'spopk.dll',
    'spp.dll',
    'sppc.dll',
    'sppcc.dll',
    'sppcext.dll',
    'sppcomapi.dll',
    'sppcommdlg.dll',
    'sppinst.dll',
    'sppwmi.dll',
    'spwinsat.dll',
    'spwizeng.dll',
    'spwizimg.dll',
    'spwizres.dll',
    'spwmp.dll',
    'sqlceoledb30.dll',
    'sqlceqp30.dll',
    'sqlcese30.dll',
    'sqlsrv32.dll',
    'sqlunirl.dll',
    'sqlwid.dll',
    'sqlwoa.dll',
    'sqmapi.dll',
    'srchadmin.dll',
    'srclient.dll',
    'srhelper.dll',
    'srpuxnativesnapin.dll',
    'srvcli.dll',
    'sscore.dll',
    'ssdpapi.dll',
    'sspicli.dll',
    'ssshim.dll',
    'stclient.dll',
    'sti.dll',
    'stobject.dll',
    'storage.dll',
    'storagecontexthandler.dll',
    'storprop.dll',
    'structuredquery.dll',
    'sud.dll',
    'sxproxy.dll',
    'sxs.dll',
    'sxshared.dll',
    'sxsstore.dll',
    'synccenter.dll',
    'synceng.dll',
    'synchostps.dll',
    'syncinfrastructure.dll',
    'syncinfrastructureps.dll',
    'syncreg.dll',
    'syncui.dll',
    'syssetup.dll',
    'systemcpl.dll',
    't2embed.dll',
    'tapi3.dll',
    'tapi32.dll',
    'tapimigplugin.dll',
    'tapiperf.dll',
    'tapisrv.dll',
    'tapisysprep.dll',
    'tapiui.dll',
    'taskcomp.dll',
    'taskschd.dll',
    'taskschdps.dll',
    'tbs.dll',
    'tcpipcfg.dll',
    'tcpmonui.dll',
    'tdh.dll',
    'termmgr.dll',
    'thawbrkr.dll',
    'themecpl.dll',
    'themeui.dll',
    'thumbcache.dll',
    'timedatemuicallback.dll',
    'tlscsp.dll',
    'tpmcompc.dll',
    'tquery.dll',
    'traffic.dll',
    'trapi.dll',
    'tsbyuv.dll',
    'tschannel.dll',
    'tsgqec.dll',
    'tsmf.dll',
    'tspkg.dll',
    'tsworkspace.dll',
    'tvratings.dll',
    'twext.dll',
    'txflog.dll',
    'txfw32.dll',
    'typelib.dll',
    'tzres.dll',
    'ubpm.dll',
    'ucmhc.dll',
    'ucrtbase.dll',
    'udhisapi.dll',
    'uexfat.dll',
    'ufat.dll',
    'uianimation.dll',
    'uiautomationcore.dll',
    'uicom.dll',
    'uiribbon.dll',
    'uiribbonres.dll',
    'ulib.dll',
    'umdmxfrm.dll',
    'unimdmat.dll',
    'uniplat.dll',
    'untfs.dll',
    'upnp.dll',
    'upnphost.dll',
    'ureg.dll',
    'url.dll',
    'urlmon.dll',
    'usbceip.dll',
    'usbperf.dll',
    'usbui.dll',
    'user32.dll',
    'useraccountcontrolsettings.dll',
    'usercpl.dll',
    'userenv.dll',
    'usp10.dll',
    'utildll.dll',
    'uudf.dll',
    'uxinit.dll',
    'uxlib.dll',
    'uxlibres.dll',
    'uxtheme.dll',
    'van.dll',
    'vault.dll',
    'vaultcli.dll',
    'vbajet32.dll',
    'vbscript.dll',
    'vdmdbg.dll',
    'vdsbas.dll',
    'vdsdyn.dll',
    'vdsvd.dll',
    'vds_ps.dll',
    'verifier.dll',
    'version.dll',
    'vfpodbc.dll',
    'vfwwdm32.dll',
    'vidreszr.dll',
    'virtdisk.dll',
    'vpnikeapi.dll',
    'vssapi.dll',
    'vsstrace.dll',
    'vss_ps.dll',
    'w32topl.dll',
    'wabsyncprovider.dll',
    'wavemsp.dll',
    'wbemcomn.dll',
    'wcnapi.dll',
    'wcncsvc.dll',
    'wcneapauthproxy.dll',
    'wcneappeerproxy.dll',
    'wcnwiz.dll',
    'wcspluginservice.dll',
    'wdc.dll',
    'wdi.dll',
    'wdigest.dll',
    'wdscore.dll',
    'webcheck.dll',
    'webclnt.dll',
    'webio.dll',
    'webservices.dll',
    'wecapi.dll',
    'wer.dll',
    'werdiagcontroller.dll',
    'werui.dll',
    'wevtapi.dll',
    'wevtfwd.dll',
    'wfapigp.dll',
    'wfhc.dll',
    'whealogr.dll',
    'whhelper.dll',
    'wiaaut.dll',
    'wiadefui.dll',
    'wiadss.dll',
    'wiaextensionhost64.dll',
    'wiascanprofiles.dll',
    'wiashext.dll',
    'wiatrace.dll',
    'wiavideo.dll',
    'wimgapi.dll',
    'win32spl.dll',
    'winbio.dll',
    'winbrand.dll',
    'wincredprovider.dll',
    'windowscodecs.dll',
    'windowscodecsext.dll',
    'winfax.dll',
    'winhttp.dll',
    'wininet.dll',
    'winipsec.dll',
    'winmm.dll',
    'winnsi.dll',
    'winrnr.dll',
    'winrscmd.dll',
    'winrsmgr.dll',
    'winrssrv.dll',
    'winsatapi.dll',
    'winscard.dll',
    'winshfhc.dll',
    'winsockhc.dll',
    'winsrpc.dll',
    'winsta.dll',
    'winsync.dll',
    'winsyncmetastore.dll',
    'winsyncproviders.dll',
    'wintrust.dll',
    'winusb.dll',
    'wkscli.dll',
    'wksprtps.dll',
    'wlanapi.dll',
    'wlancfg.dll',
    'wlanconn.dll',
    'wlandlg.dll',
    'wlangpui.dll',
    'wlanhlp.dll',
    'wlaninst.dll',
    'wlanmm.dll',
    'wlanmsm.dll',
    'wlanpref.dll',
    'wlansec.dll',
    'wlanui.dll',
    'wlanutil.dll',
    'wldap32.dll',
    'wlgpclnt.dll',
    'wls0wndh.dll',
    'wmadmod.dll',
    'wmadmoe.dll',
    'wmasf.dll',
    'wmcodecdspps.dll',
    'wmdmlog.dll',
    'wmdmps.dll',
    'wmdrmdev.dll',
    'wmdrmnet.dll',
    'wmdrmsdk.dll',
    'wmerror.dll',
    'wmi.dll',
    'wmidx.dll',
    'wmiprop.dll',
    'wmnetmgr.dll',
    'wmp.dll',
    'wmpcm.dll',
    'wmpdui.dll',
    'wmpdxm.dll',
    'wmpeffects.dll',
    'wmpencen.dll',
    'wmphoto.dll',
    'wmploc.dll',
    'wmpmde.dll',
    'wmpps.dll',
    'wmpshell.dll',
    'wmpsrcwp.dll',
    'wmsgapi.dll',
    'wmspdmod.dll',
    'wmspdmoe.dll',
    'wmvcore.dll',
    'wmvdecod.dll',
    'wmvdspa.dll',
    'wmvencod.dll',
    'wmvsdecd.dll',
    'wmvsencd.dll',
    'wmvxencd.dll',
    'wow32.dll',
    'wpc.dll',
    'wpcao.dll',
    'wpcsvc.dll',
    'wpdshext.dll',
    'wpdshserviceobj.dll',
    'wpdsp.dll',
    'wpdwcn.dll',
    'ws2help.dll',
    'ws2_32.dll',
    'wscapi.dll',
    'wscinterop.dll',
    'wscisvif.dll',
    'wscmisetup.dll',
    'wscproxystub.dll',
    'wsdapi.dll',
    'wsdchngr.dll',
    'wsecedit.dll',
    'wshbth.dll',
    'wshcon.dll',
    'wshelper.dll',
    'wshext.dll',
    'wship6.dll',
    'wshirda.dll',
    'wshqos.dll',
    'wshrm.dll',
    'wshtcpip.dll',
    'wsmanmigrationplugin.dll',
    'wsmauto.dll',
    'wsmplpxy.dll',
    'wsmres.dll',
    'wsmsvc.dll',
    'wsmwmipl.dll',
    'wsnmp32.dll',
    'wsock32.dll',
    'wtsapi32.dll',
    'wuapi.dll',
    'wudriver.dll',
    'wups.dll',
    'wuwebv.dll',
    'wvc.dll',
    'wwanapi.dll',
    'wwapi.dll',
    'wzcdlg.dll',
    'xinput9_1_0.dll',
    'xmlfilter.dll',
    'xmllite.dll',
    'xmlprovi.dll',
    'xolehlp.dll',
    'xpsfilt.dll',
    'xpsgdiconverter.dll',
    'xpsprint.dll',
    'xpsrasterservice.dll',
    'xpsservices.dll',
    'xpsshhdr.dll',
    'xpssvcs.dll',
    'xwizards.dll',
    'xwreg.dll',
    'xwtpdui.dll',
    'xwtpw32.dll',
    'zipfldr.dll',
    'msacm32.drv',
    'wdmaud.drv',
    'winspool.drv',
}

# set of regular expressions for additional DLLs to ignore
ignore_regexes = {
    re.compile(r'^python[0-9]+\.dll$'),  # included in Python distribution
    re.compile(r'^api-'),  # let Windows handle API sets
}

# Prefixes of DLLs whose names should not be mangled. These either are
# dependencies of DLLs that contain data after the PE file proper (and thus
# cannot be modified by machomachomangler) or already have the version in the
# filename.
no_mangle_prefixes = {
    'vcruntime',
    'vccorlib',
    'msvcp',
    'msvcr',
    'concrt',
    'mfc',
    'vcamp',
    'vcomp',
}



"""DLL file patching functions."""

import itertools
import os
import platform
import ctypes.util
import typing
import pefile
import machomachomangler.pe
from . import dll_list


class PEContext:
    """Context manager for PE file."""
    def __init__(self, name: str) -> None:
        self._pe = pefile.PE(name)

    def __enter__(self) -> pefile.PE:
        return self._pe

    def __exit__(self, exc_type, exc_value, traceback):
        self._pe.close()


def get_direct_needed(lib_path: str, include_delay_imports: bool = True) -> set:
    """Given the path to a shared library, return a set containing the lowercase
    DLL names of all its direct dependencies.
    If include_delay_imports is True, delay-loaded dependencies are included.
    Otherwise, they are not included"""
    with PEContext(lib_path) as pe:
        imports = []
        if include_delay_imports:
            attrs = ('DIRECTORY_ENTRY_IMPORT', 'DIRECTORY_ENTRY_DELAY_IMPORT')
        else:
            attrs = ('DIRECTORY_ENTRY_IMPORT',)
        for attr in attrs:
            if hasattr(pe, attr):
                imports = itertools.chain(imports, getattr(pe, attr))
        needed = set()
        for entry in imports:
            needed.add(entry.dll.decode('utf-8').lower())
    return needed


def get_direct_mangleable_needed(lib_path: str, no_dlls: set, no_mangles: set) -> set:
    """Given the path to a shared library, return a set containing the lowercase
    DLL names of all direct dependencies that belong in the wheel and should be
    name-mangled.

    no_dlls is a set of lowercase additional DLL names that do not belong in the
    wheel.

    no_mangles is a set of lowercase additional DLL names not to mangle."""
    with PEContext(lib_path) as pe:
        imports = []
        for attr in ('DIRECTORY_ENTRY_IMPORT', 'DIRECTORY_ENTRY_DELAY_IMPORT'):
            if hasattr(pe, attr):
                imports = itertools.chain(imports, getattr(pe, attr))
        needed = set()
        lib_bitness = 64 if pe.FILE_HEADER.Machine == 34404 else 32
        ignore_names = dll_list.ignore_names_32 if lib_bitness == 32 else dll_list.ignore_names_64
        for entry in imports:
            dll_name = entry.dll.decode('utf-8').lower()
            if dll_name not in ignore_names and \
                    dll_name not in no_dlls and \
                    not any(r.search(dll_name) for r in dll_list.ignore_regexes) and \
                    dll_name not in no_mangles and \
                    not any(dll_name.startswith(prefix) for prefix in dll_list.no_mangle_prefixes):
                needed.add(dll_name)
    return needed


def get_all_needed(lib_path: str,
                   add_dlls: set,
                   no_dlls: set,
                   on_error: str = 'raise') -> typing.Tuple[typing.Set[str], typing.Set[str], typing.Set[str]]:
    """Given the path to a shared library, return a 3-tuple of sets
    (discovered, ignored, not_found).

    discovered contains the DLL paths of all direct and indirect dependencies of
    that shared library that should be bundled into the wheel. ignored contains
    the DLL names of all direct and indirect dependencies of that shared library
    that will not be bundled into the wheel because they are assumed to be on
    the target system.

    If on_error is 'raise', FileNotFoundError is raised if a dependent library
    cannot be found, and not_found is the empty set. If on_error is 'ignore',
    not_found contains the DLL names of all dependent DLLs that cannot be
    found.

    All DLL names in the returned sets are lowercase.

    add_dlls is a set of DLL names to force inclusion into the wheel. We do not
    search for dependencies of these DLLs.

    no_dlls is a set of DLL names to force exclusion from the wheel. We do not
    search for dependencies of these DLLs. Cannot overlap with add_dlls.
    """
    first_lib_path = lib_path.lower()
    stack = [first_lib_path]
    discovered = set()
    ignored = set()
    not_found = set()
    while stack:
        lib_path = stack.pop()
        if lib_path not in discovered:
            discovered.add(lib_path)
            interpreter_bitness = 64 if platform.architecture()[0] == '64bit' else 32
            with PEContext(lib_path) as pe:
                lib_bitness = 64 if pe.FILE_HEADER.Machine == 34404 else 32
                if interpreter_bitness != lib_bitness:
                    # bitness of Python interpreter must match that of the DLL
                    # so that ctypes.util.find_library() can find it
                    raise OSError(f'Dependent library {lib_path} is {lib_bitness}-bit but Python interpreter is {interpreter_bitness}-bit')

                imports = []
                for attr in ('DIRECTORY_ENTRY_IMPORT', 'DIRECTORY_ENTRY_DELAY_IMPORT'):
                    if hasattr(pe, attr):
                        imports = itertools.chain(imports, getattr(pe, attr))
                ignore_names = dll_list.ignore_names_32 if lib_bitness == 32 else dll_list.ignore_names_64
                for entry in imports:
                    dll_name = entry.dll.decode('utf-8').lower()
                    if dll_name not in ignore_names and \
                            not any(r.search(dll_name) for r in dll_list.ignore_regexes) and \
                            dll_name not in no_dlls:
                        dll_path = ctypes.util.find_library(dll_name)
                        if dll_path:
                            stack.append(dll_path)
                        elif on_error == 'raise':
                            raise FileNotFoundError(f'Unable to find library: {dll_name}')
                        else:
                            not_found.add(dll_name)
                    elif dll_name not in add_dlls:
                        ignored.add(dll_name)
    discovered.remove(first_lib_path)
    for add_dll_name in add_dlls:
        add_dll_path = ctypes.util.find_library(add_dll_name)
        if add_dll_path:
            discovered.add(add_dll_path)
        elif on_error == 'raise':
            raise FileNotFoundError(f'Unable to find library: {add_dll_name}')
        else:
            not_found.add(add_dll_name)
    return discovered, ignored, not_found


def replace_needed(lib_path: str, old_deps: typing.Iterable, name_map: dict) -> None:
    """For the DLL at lib_path, replace its declared dependencies on old_deps
    with those in name_map.
    old_deps: a subset of the dependencies that lib_path has
    name_map: a dict that maps an old dependency name to a new name, must
        contain at least all the keys in old_deps"""
    used_name_map = {dep.encode('utf-8'): name_map[dep].encode('utf-8') for dep in old_deps}
    if not used_name_map:
        # no dependency names to change
        return
    with open(lib_path, 'rb') as f:
        buf = f.read()
    try:
        buf = machomachomangler.pe.redll(buf, used_name_map)
    except ValueError as ex:
        if "Can't add new section" in str(ex):
            raise RuntimeError(
                'Unable to rename the dependencies of '
                f'{os.path.basename(lib_path)} because this DLL has trailing '
                'data. If this DLL was created with MinGW, run the strip '
                f'utility. Otherwise, include {old_deps} in the --no-mangle '
                'flag. In addition, if you believe that delvewheel should '
                'avoid name-mangling a specific DLL by default, open an issue '
                'at https://github.com/adang1345/delvewheel/issues and include '
                'this error message.')
        raise ex
    with open(lib_path, 'wb') as f:
        f.write(buf)
    with PEContext(lib_path) as pe:
        pe.OPTIONAL_HEADER.CheckSum = pe.generate_checksum()
        pe.write(lib_path)



import setuptools
import shutil


def rmtree(path):
    """Like shutil.rmtree() but is silent if path does not exist."""
    try:
        shutil.rmtree(path)
    except FileNotFoundError:
        pass


class CleanCommand(setuptools.Command):
    """Clean all build and distribution files"""
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        rmtree('build')
        rmtree('dist')
        rmtree('delvewheel.egg-info')
        rmtree('delvewheel/__pycache__')


setuptools.setup(cmdclass={'clean': CleanCommand})



import argparse
import os
import sys
from .wheel_repair import WheelRepair
from . import patch_dll


def subdir_suffix(s: str) -> str:
    """Helper for argument parser for validating a subdirectory suffix."""
    if not s or any(c in r'<>:"/\|?*' or ord(c) < 32 for c in s) or \
            any(s.endswith(x) for x in ('.dist-info', '.data', ' ', '.')):
        raise argparse.ArgumentTypeError(f'Invalid subdirectory suffix {s!r}')
    return s


def main():
    """Main function"""
    # disallow usage on non-Windows platforms
    if sys.platform != 'win32':
        raise EnvironmentError('delvewheel is supported on Windows only')

    # parse arguments
    parser = argparse.ArgumentParser(description='Self-contained wheels for Windows')
    kwargs = {'dest': 'command'}
    if sys.version_info[:2] >= (3, 7):
        kwargs['required'] = True
    subparsers = parser.add_subparsers(**kwargs)
    parser_show_description = 'Search a wheel for external DLL dependencies'
    parser_show = subparsers.add_parser('show', help=parser_show_description, description=parser_show_description)
    parser_repair_description = 'Vendor in external DLL dependencies of a wheel'
    parser_repair = subparsers.add_parser('repair', help=parser_repair_description, description=parser_repair_description)
    parser_needed_description = 'Show the direct DLL dependencies of a file'
    parser_needed = subparsers.add_parser('needed', help=parser_needed_description, description=parser_needed_description)
    for subparser in (parser_show, parser_repair):
        subparser.add_argument('wheel', nargs='+', help='wheel(s) to show or repair')
        subparser.add_argument('--add-path', default='', metavar='PATHS', help='extra paths(s) to search for DLLs, semicolon-delimited')
        subparser.add_argument('--add-dll', default='', metavar='DLLS', help='force inclusion of DLL name(s), semicolon-delimited')
        subparser.add_argument('--no-dll', default='', metavar='DLLS', help='force exclusion of DLL name(s), semicolon-delimited')
        subparser.add_argument('-v', action='count', default=0, help='verbose mode')
        subparser.add_argument('--extract-dir', help=argparse.SUPPRESS)
    parser_repair.add_argument('-w', '--wheel-dir', dest='target', default='wheelhouse', help='directory to write repaired wheel')
    parser_repair.add_argument('--no-mangle', default='', metavar='DLLS', help='DLL names(s) not to mangle, semicolon-delimited')
    parser_repair.add_argument('--no-mangle-all', action='store_true', help="don't mangle any DLL names")
    parser_repair.add_argument('-L', '--lib-sdir', default='.libs', type=subdir_suffix, help='directory suffix in package to store vendored DLLs (default .libs)')
    parser_needed.add_argument('file', help='path to a DLL file')
    args = parser.parse_args()
    if args.command is None:
        raise ValueError('No command provided. Use -h for help.')

    # handle command
    if args.command in ('show', 'repair'):
        add_paths = set(os.path.abspath(path) for path in args.add_path.split(';') if path)
        add_dlls = set(dll_name.lower() for dll_name in args.add_dll.split(';') if dll_name)
        no_dlls = set(dll_name.lower() for dll_name in args.no_dll.split(';') if dll_name)

        intersection = add_dlls & no_dlls
        if intersection:
            raise ValueError(f'Cannot force both inclusion and exclusion of {intersection}')

        if add_paths:
            os.environ['PATH'] = f'{os.pathsep.join(add_paths)}{os.pathsep}{os.environ["PATH"]}'

        for wheel in args.wheel:
            wr = WheelRepair(wheel, args.extract_dir, add_dlls, no_dlls, args.v)
            if args.command == 'show':
                wr.show()
            else:  # args.command == 'repair'
                no_mangles = set(dll_name.lower() for dll_name in args.no_mangle.split(';') if dll_name)
                wr.repair(args.target, no_mangles, args.no_mangle_all, args.lib_sdir)
    else:  # args.command == 'needed'
        for dll_name in patch_dll.get_direct_needed(args.file):
            print(dll_name)


if __name__ == '__main__':
    main()
